# -*- coding: utf-8 -*-
"""
Author:
    Bjarne S. Kallesoee


Description:
    Reads all HAWC2 output data formats, HAWC2 ascii, HAWC2 binary and FLEX

call ex.:
    # creat data file object, call without extension, but with parth
    file = ReadHawc2("HAWC2ex/test")
    # if called with ReadOnly = 1 as
    file = ReadHawc2("HAWC2ex/test",ReadOnly=1)
    # no channels a stored in memory, otherwise read channels are stored for reuse

    # channels are called by a list
    file([0,2,1,1])  => channels 1,3,2,2
    # if empty all channels are returned
    file()  => all channels as 1,2,3,...
    file.t => time vector

1. version: 19/4-2011
2. version: 5/11-2015 fixed columns to get description right, fixed time vector (mmpe@dtu.dk)

Need to be done:
    * add error handling for allmost every thing

"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from builtins import int
from builtins import range
from io import open as opent
from builtins import str
from future import standard_library
standard_library.install_aliases()
from builtins import object
import numpy as np
import os

from wetb import gtsdf

# FIXME: numpy doesn't like io.open binary fid in PY27, why is that? As a hack
# workaround, use opent for PY23 compatibility when handling text files,
# and default open for binary

################################################################################
################################################################################
################################################################################
## Read HAWC2 class
################################################################################
class ReadHawc2(object):
    """
    """
################################################################################
# read *.sel file
    def _ReadSelFile(self):
        """
        Some title
        ==========

        Using docstrings formatted according to the reStructuredText specs
        can be used for automated documentation generation with for instance
        Sphinx: http://sphinx.pocoo.org/.

        Parameters
        ----------
        signal : ndarray
            some description

        Returns
        -------
        output : int
            describe variable
        """

        # read *.sel hawc2 output file for result info
        if self.FileName.lower().endswith('.sel'):
            self.FileName = self.FileName[:-4]
        fid = opent(self.FileName + '.sel', 'r')
        Lines = fid.readlines()
        fid.close()
        # findes general result info (number of scans, number of channels,
        # simulation time and file format)
        temp = Lines[8].split()
        self.NrSc = int(temp[0])
        self.NrCh = int(temp[1])
        self.Time = float(temp[2])
        self.Freq = self.NrSc / self.Time
        self.t = np.linspace(0, self.Time, self.NrSc + 1)[1:]
        Format = temp[3]
        # reads channel info (name, unit and description)
        Name = []; Unit = []; Description = [];
        for i in range(0, self.NrCh):
            temp = str(Lines[i + 12][12:43]); Name.append(temp.strip())
            temp = str(Lines[i + 12][43:54]); Unit.append(temp.strip())
            temp = str(Lines[i + 12][54:-1]); Description.append(temp.strip())
        self.ChInfo = [Name, Unit, Description]
        # if binary file format, scaling factors are read
        if Format.lower() == 'binary':
            self.ScaleFactor = np.zeros(self.NrCh)
            self.FileFormat = 'HAWC2_BINARY'
            for i in range(0, self.NrCh):
                self.ScaleFactor[i] = float(Lines[i + 12 + self.NrCh + 2])
        else:
            self.FileFormat = 'HAWC2_ASCII'
################################################################################
# read sensor file for FLEX format
    def _ReadSensorFile(self):
        # read sensor file used if results are saved in FLEX format
        DirName = os.path.dirname(self.FileName + ".int")
        try:
            fid = opent(DirName + "\sensor ", 'r')
        except IOError:
            print ("can't finde sensor file for FLEX format")
            return
        Lines = fid.readlines()
        fid.close()
        # reads channel info (name, unit and description)
        self.NrCh = 0
        Name = []; Unit = []; Description = [];
        for i in range(2, len(Lines)):
            temp = Lines[i]
            if not temp.strip():
                break
            self.NrCh += 1
            temp = str(Lines[i][38:45]); Unit.append(temp.strip())
            temp = str(Lines[i][45:53]); Name.append(temp.strip())
            temp = str(Lines[i][53:]); Description.append(temp.strip())
        self.ChInfo = [Name, Unit, Description]
        # read general info from *.int file
        fid = open(self.FileName + ".int", 'rb')
        fid.seek(4 * 19)
        if not np.fromfile(fid, 'int32', 1) == self.NrCh:
            print ("number of sensors in sensor file and data file are not consisten")
        fid.seek(4 * (self.NrCh) + 8, 1)
        temp = np.fromfile(fid, 'f', 2)
        self.Freq = 1 / temp[1];
        self.ScaleFactor = np.fromfile(fid, 'f', self.NrCh)
        fid.seek(2 * 4 * self.NrCh + 48 * 2)
        self.NrSc = int(len(np.fromfile(fid, 'int16')) / self.NrCh)
        self.Time = self.NrSc * temp[1]
        self.t = np.arange(0, self.Time, temp[1])
        fid.close()
################################################################################
# init function, load channel and other general result file info
    def __init__(self, FileName, ReadOnly=0):
        self.FileName = FileName
        self.ReadOnly = ReadOnly
        self.Iknown = []  # to keep track of what has been read all ready
        self.Data = np.zeros(0)
        if FileName.lower().endswith('.sel') or os.path.isfile(FileName + ".sel"):
             self._ReadSelFile()
        elif FileName.lower().endswith('.int') or os.path.isfile(self.FileName + ".int"):
             self.FileFormat = 'FLEX'
             self._ReadSensorFile()
        elif FileName.lower().endswith('.hdf5') or os.path.isfile(self.FileName + ".hdf5"):
            self.FileFormat = 'GTSDF'
            self.ReadGtsdf()
        else:
            print ("unknown file: " + FileName)
################################################################################
# Read results in binary format
    def ReadBinary(self, ChVec=[]):
        if not ChVec:
            ChVec = range(0, self.NrCh)
        with open(self.FileName + '.dat', 'rb') as fid:
            data = np.zeros((self.NrSc, len(ChVec))); j = 0
            for i in ChVec:
                fid.seek(i * self.NrSc * 2, 0)
                data[:, j] = np.fromfile(fid, 'int16', self.NrSc) * self.ScaleFactor[i]
                j += 1
        return data
################################################################################
# Read results in ASCII format
    def ReadAscii(self, ChVec=[]):
        if not ChVec:
            ChVec = range(0, self.NrCh)
        temp = np.loadtxt(self.FileName + '.dat', usecols=ChVec)
        return temp.reshape((self.NrSc, len(ChVec)))
################################################################################
# Read results in FLEX format
    def ReadFLEX(self, ChVec=[]):
        if not ChVec:
            ChVec = range(1, self.NrCh)
        fid = open(self.FileName + ".int", 'rb')
        fid.seek(2 * 4 * self.NrCh + 48 * 2)
        temp = np.fromfile(fid, 'int16')
        temp = temp.reshape(self.NrSc, self.NrCh)
        fid.close()
        return np.dot(temp[:, ChVec], np.diag(self.ScaleFactor[ChVec]))
################################################################################
# Read results in GTSD format
    def ReadGtsdf(self):
        self.t, data, info = gtsdf.load(self.FileName + '.hdf5')
        self.Time = self.t
        self.ChInfo = [['Time'] + info['attribute_names'],
                       ['s'] + info['attribute_units'],
                       ['Time'] + info['attribute_descriptions']]
        self.NrCh = data.shape[1] + 1
        self.NrSc = data.shape[0]
        self.Freq = self.NrSc / self.Time
        self.FileFormat = 'GTSDF'
        self.gtsdf_description = info['description']
        data = np.hstack([self.Time[:,np.newaxis], data])
        return data
################################################################################
# One stop call for reading all data formats
    def ReadAll(self, ChVec=[]):
        if not ChVec and not self.FileFormat == 'GTSDF':
            ChVec = range(0, self.NrCh)
        if self.FileFormat == 'HAWC2_BINARY':
            return self.ReadBinary(ChVec)
        elif self.FileFormat == 'HAWC2_ASCII':
            return self.ReadAscii(ChVec)
        elif self.FileFormat == 'GTSDF':
            return self.ReadGtsdf()
        else:
            return self.ReadFLEX(ChVec)
################################################################################
# Main read data call, read, save and sort data
    def __call__(self, ChVec=[]):
        if not ChVec:
            ChVec = range(0, self.NrCh)
        elif max(ChVec) >= self.NrCh:
            print ("to high channel number")
            return
        # if ReadOnly, read data but no storeing in memory
        if self.ReadOnly:
            return self.ReadAll(ChVec)
        # if not ReadOnly, sort in known and new channels, read new channels
        # and return all requested channels
        else:
            # sort into known channels and channels to be read
            I1 = [];I2 = []  # I1=Channel mapping, I2=Channels to be read
            for i in ChVec:
                try:
                    I1.append(self.Iknown.index(i))
                except:
                    self.Iknown.append(i)
                    I2.append(i)
                    I1.append(len(I1))
            # read new channels
            if I2:
                temp = self.ReadAll(I2)
                # add new channels to Data
                if self.Data.any():
                    self.Data = np.append(self.Data, temp, axis=1)
                # if first call, so Daata is empty
                else:
                    self.Data = temp
            return self.Data[:, tuple(I1)]


################################################################################
################################################################################
################################################################################
## write HAWC2 class, to be implemented
################################################################################

if __name__ == '__main__':
    res_file = ReadHawc2('structure_wind')
    results = res_file.ReadAscii()
    channelinfo = res_file.ChInfo



'''
Created on 02/10/2015

@author: MMPE
'''
import unittest

import numpy as np
from wetb.wind.turbulence.constraint_file import ConstraintFile
from wetb.utils.test_files import get_test_file
class TestConstraintFile(unittest.TestCase):


    def testConstraintFile(self):
        time = [0, 1, 2]
        u = [5, 7, 9]
        v = [1.5, 1, .5]
        w = [3, 1, 5]
        tu = np.array([time, u]).T
        tuvw = np.array([time, u, v, w]).T
        glpos = (0, 0, -85)
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 0, -90), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints(glpos, tu)
        np.testing.assert_array_equal(constraint_file.constraints['u'][1], (6, 4, 4, 0))
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 0, -90), box_transport_speed=5, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints(glpos, tuvw)
        np.testing.assert_array_equal(constraint_file.constraints['u'][2], (6, 4, 4, 2))
        np.testing.assert_array_equal(constraint_file.constraints['v'][2], (6, 4, 4, -.5))
        np.testing.assert_array_equal(constraint_file.constraints['w'][2], (6, 4, 4, 2))
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 0, -90), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints((-10, -4, -95), tu)
        np.testing.assert_array_equal(constraint_file.constraints['u'][1], (8, 5, 5, 0))
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 4, -90), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints((-10, 0, -95), tu)
        np.testing.assert_array_equal(constraint_file.constraints['u'][1], (8, 5, 5, 0))
        #constraint_file.save("./test_files", "constraints_test",'')

    def testConstraintFile_Mxyz(self):
        time = [0]
        u = [5]
        tu = np.array([time, u]).T

        glpos = (0, 0, -85)
        constraint_file = ConstraintFile(center_gl_xyz=(0, 0, 0), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints([0,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 5, 5, 5))
        constraint_file.add_constraints([5,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 4, 5, 5))
        constraint_file.add_constraints([0,0,5], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 5, 4, 5))
        constraint_file.add_constraints([0,-10,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (6, 5, 5, 5))
        
    def testConstraintFile_center_gl_xyz(self):
        time = [0]
        u = [5]
        tu = np.array([time, u]).T

        constraint_file = ConstraintFile(center_gl_xyz=(0, 0, 0), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints([0,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 5, 5, 5))
        
        constraint_file = ConstraintFile(center_gl_xyz=(0, 0, -5), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints([0,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 5, 4, 5))
        
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 0, 0), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints([0,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (1, 4, 5, 5))
        
        constraint_file = ConstraintFile(center_gl_xyz=(0, 10, 0), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.add_constraints([0,0,0], tu, subtract_mean=False)
        np.testing.assert_array_equal(constraint_file.constraints['u'][-1], (6, 5, 5, 5))
        

    def test_outputbox_errors(self):
        time = [0, 1, 2]
        u = [5, 7, 9]
        v = [1.5, 1, .5]
        w = [3, 1, 5]
        tu = np.array([time, u]).T
        tuvw = np.array([time, u, v, w]).T
        glpos = (0, 0, -85)
        constraint_file = ConstraintFile(center_gl_xyz=(0, 0, -85), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        self.assertRaisesRegex(ValueError, "At time, t=0, global position \(0,0,-44\)", constraint_file.add_constraints, (0, 0, -85 + 35 + 6), tu)
        self.assertRaisesRegex(ValueError, "At time, t=0, global position \(0,0,-126\)", constraint_file.add_constraints, (0, 0, -85 - 35 - 6), tu)
        self.assertRaisesRegex(ValueError, "At time, t=0, global position \(-41,0,-85\)", constraint_file.add_constraints, (-35 - 6, 0, -85), tu)
        self.assertRaisesRegex(ValueError, "At time, t=0, global position \(41,0,-85\)", constraint_file.add_constraints, (35 + 6, 0, -85), tu)
        self.assertRaisesRegex(ValueError, "At time, t=0, global position \(0,2,-85\)", constraint_file.add_constraints, (0, 2, -85), tu)
        self.assertRaisesRegex(ValueError, "At time, t=2, global position \(0,-13,-85\)", constraint_file.add_constraints, (0, -13, -85), tu)

    def test_hawc2_cmd(self):
        constraint_file = ConstraintFile(center_gl_xyz=(0, 0, -85), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        mann = constraint_file.hawc2_mann_section("test",1)
        self.assertEqual(mann.box_dim_u.values, [16,1.875])
        
    def test_load(self):
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 4, -90), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.load(get_test_file('constraints_test.con'))
        with open(get_test_file('constraints_test.con')) as fid:
            self.assertEqual(str(constraint_file), fid.read())
        
    def test_time_series(self):
        constraint_file = ConstraintFile(center_gl_xyz=(-5, 4, -90), box_transport_speed=10, no_grid_points=(16, 8, 8), box_size=(30, 70, 70))
        constraint_file.load(get_test_file('constraints_test.con'))
        time,uvw = constraint_file.time_series(-4)
        u,v,w = uvw.T
        m = ~np.isnan(u)
        np.testing.assert_array_equal(time[m],[0,1,2])
        np.testing.assert_array_equal(u[m],[-2,0,2])
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testConstraintFile']
    unittest.main()



'''
Created on 27. mar. 2017

@author: mmpe
'''
import numpy as np
from scipy import signal

def sine_generator(sample_frq, sine_frq, duration):
    """Create a sine signal for filter test
    
    Parameters
    ----------
    sample_frq : int, float
        Sample frequency of returned signal [Hz]
    sine_frq : int, float
        Frequency of sine signal [Hz]
    duration : int, float
        Duration of returned signal [s]
        
    Returns
    -------
    t,sig : ndarray, ndarray
        time (t) and sine signal (sig)
    
    Examples
    --------
    >>> sine_generator(10,1,2) 
    """
    T = duration
    nsamples = sample_frq * T
    w = 2. * np.pi * sine_frq
    t = np.linspace(0, T, nsamples, endpoint=False)
    sig = np.sin(w * t)
    return t, sig


def low_pass(x, sample_frq, cutoff_frq, order=5):
    """Low pass filter (butterworth)
    
    Parameters
    ----------
    x : array_like
        Input signal
    sample_frq : int, float
        Sample frequency [Hz]
    cutoff_frq : int, float
        Cut off frequency [Hz]
    order : int
        Order of the filter (1th order: 20db per decade, 2th order:)
    
    Returns
    -------
    y : ndarray
        Low pass filtered signal
    
    
    Examples
    --------
    >>> 
    """
    nyquist_frq = 0.5 * sample_frq
    normal_cutoff = cutoff_frq / nyquist_frq
    b,a = signal.butter(order, normal_cutoff, btype='low', analog=False)
    return signal.filtfilt(b, a, x)

def high_pass(x, sample_frq, cutoff_frq, order=5):
    """Low pass filter (butterworth)
    
    Parameters
    ----------
    x : array_like
        Input signal
    sample_frq : int, float
        Sample frequency [Hz]
    cutoff_frq : int, float
        Cut off frequency [Hz]
    order : int
        Order of the filter (1th order: 20db per decade, 2th order:)
    
    Returns
    -------
    y : ndarray
        Low pass filtered signal
    
    
    Examples
    --------
    >>> 
    """
    nyquist_frq = 0.5 * sample_frq
    normal_cutoff = cutoff_frq / nyquist_frq
    b,a = signal.butter(order, normal_cutoff, btype='high', analog=False)
    return signal.filtfilt(b, a, x)

def frequency_response(sample_frq, cutoff_frq, type, order, plt=None):
    """Frequency response of low/high pass filter (butterworth
    
    Parameters
    ----------
    sample_frq : int, float
        Sample frequency [Hz]
    cutoff_frq : int, float
        Cut off frequency [Hz]
    type : {'low','high'}
        Low or high pass filter
    order : int
        Order of the filter (1th order: 20db per decade, 2th order: 40db per decade)
    plt : pyplot, optional
        If specified, the frequency response is plotted
    
    Returns
    -------
    w,h : ndarray, ndarray
        Frequency (w) in Hz and filter response in db
    
    
    Examples
    --------
    >>>  
    """
    nyquist_frq = 0.5 * sample_frq
    normal_cutoff = cutoff_frq / nyquist_frq
    assert 0<normal_cutoff<1, "cutoff frequency must be <= nyquist frequency"
    b,a = signal.butter(order, cutoff_frq, btype=type, analog=True)
    w, h = signal.freqs(b, a)
    h_db = 20 * np.log10(abs(h))
    if plt:
        plt.plot(w, h_db, label='%d order filter response'%order)

        plt.legend(loc=0)
        
        title = 'Butterworth filter frequency response'
        if plt.axes().get_title()!=title:
            plt.title(title)
            plt.xlabel('Frequency [Hz]')
            plt.ylabel('Amplitude [dB]')
            plt.margins(.1, .1)
            plt.xscale('log')
            
            plt.grid(which='both', axis='both')
            plt.axvline(cutoff_frq, color='green') # cutoff frequency
        
    return w,h_db


'''
Created on 17/07/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest
from wetb.hawc2.sel_file import SelFile, BINARY, ASCII
from datetime import datetime
import os
testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/hawc2io/')  # test file path

class TestSelFile(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)


    def test_sel_file_ascii(self):
        sf = SelFile(testfilepath + "Hawc2ascii.sel")
        self.assertEqual(sf.version_id, "HAWC2AERO 2.4w")
        self.assertEqual(sf.created, datetime(2013, 1, 24, 10, 2, 19))
        self.assertEqual(sf.result_file, "Hawc2ascii.dat")
        self.assertEqual(sf.scans, 800)
        self.assertEqual(sf.channels, 28)
        self.assertEqual(sf.no_sensors, 28)
        self.assertEqual(sf.duration, 20)
        self.assertEqual(sf.time, 20)
        self.assertEqual(sf.format, ASCII)
        self.assertEqual(sf.sensors[0], (1, 'Time', 's', 'Time'))
        self.assertEqual(sf.sensors[1], (2, 'WSP gl. coo.,Vy', 'm/s', 'Free wind speed Vy, gl. coo, of gl. pos    2.50,  -1.00, -47.50'))


    def test_sel_file_bin(self):
        sf = SelFile(testfilepath + "Hawc2bin.sel")
        self.assertEqual(sf.version_id, "HAWC2AERO 2.4w")
        self.assertEqual(sf.created, datetime(2013, 1, 24, 10, 4, 37))
        self.assertEqual(sf.result_file, "Hawc2bin.dat")
        self.assertEqual(sf.scans, 800)
        self.assertEqual(sf.channels, 28)
        self.assertEqual(sf.no_sensors, 28)
        self.assertEqual(sf.duration, 20)
        self.assertEqual(sf.time, 20)
        self.assertEqual(sf.format, BINARY)
        self.assertEqual(sf.sensors[0], (1, 'Time', 's', 'Time'))
        self.assertEqual(sf.sensors[1], (2, 'WSP gl. coo.,Vy', 'm/s', 'Free wind speed Vy, gl. coo, of gl. pos    2.50,  -1.00, -47.50'))
        self.assertEqual(sf.scale_factors[0], 6.25000E-04)
        self.assertEqual(sf.scale_factors[1], 5.65540E-02)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 03/09/2015

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest

from wetb.fast.fast_io import load_output, load_binary_output
import os

testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path


class TestFastIO(unittest.TestCase):

    def test_load_output(self):
        data, info = load_output(testfilepath + 'DTU10MW.out')
        self.assertAlmostEqual(data[4, 3], 4.295E-04)
        self.assertEqual(info['name'], "DTU10MW")
        self.assertEqual(info['attribute_names'][1], "RotPwr")
        self.assertEqual(info['attribute_units'][1], "kW")

    def test_load_binary(self):
        data, info = load_output(testfilepath + 'test_binary.outb')
        self.assertEqual(info['name'], 'test_binary')
        self.assertEqual(info['description'], 'Modified by mwDeriveSensors on 27-Jul-2015 16:32:06')
        self.assertEqual(info['attribute_names'][4], 'RotPwr')
        self.assertEqual(info['attribute_units'][7], 'deg/s^2')
        self.assertAlmostEqual(data[10, 4], 138.822277739535)

    def test_load_binary2(self):
        # The old method was not using a buffer and was also memory expensive
        # Now use_buffer is set to true by default
        import numpy as np
        data, info = load_binary_output(testfilepath + 'test_binary.outb', use_buffer=True)
        data_old, info_old = load_binary_output(testfilepath + 'test_binary.outb', use_buffer=False)
        self.assertEqual(info['name'], info_old['name'])
        np.testing.assert_array_equal(data[0, :], data_old[0, :])
        np.testing.assert_array_equal(data[-1, :], data_old[-1, :])

    def test_load_output2(self):
        data, info = load_output(testfilepath + 'DTU10MW.out')
        self.assertEqual(info['name'], "DTU10MW")
        self.assertEqual(info['attribute_names'][1], "RotPwr")
        self.assertEqual(info['attribute_units'][1], "kW")

    def test_load_output3(self):
        # This file has an extra comment at the end
        data, info = load_output(testfilepath + 'FASTOut_Hydro.out')
        self.assertAlmostEqual(data[3, 1], -1.0E+01)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testload_output']
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 15 18:34:15 2016

@author: shfe

Description: This script is used for writing the hydro input files for HAWC2
(the wave type det_airy is not included)
"""

import os

class hydro_input(object):

    """ Basic class aiming for write the hydrodynamics input file"""

    def __init__(self, wavetype, wdepth, spectrum, Hs, Tp, seed, gamma=3.3,
                 stretching=1, wn=None, coef=200, spreading=None,
                 embed_sf=None, embed_sf_t0=None):

        self.wdepth = wdepth
        if self.wdepth < 0:
            raise ValueError('water depth must be greater than 0')

        # Regular Airy Wave Input
        if wavetype == 'reg_airy':
            self.waveno = 0
            self.argument = 'begin %s ;\n\t\tstretching %d;\n\t\twave %.2f %.2f;\n\tend;' \
                            %(wavetype,stretching,Hs,Tp)

        # Iregular Airy Wave Input
        if wavetype == 'ireg_airy':
            self.waveno = 1
            # jonswap spectrum
            if spectrum == 'jonswap':
                spectrumno = 1
                self.argument = 'begin %s ;\n\t\tstretching %d;\n\t\tspectrum %d;'\
                                '\n\t\tjonswap %.2f %.2f %.1f;\n\t\tcoef %d %d;' \
                                %(wavetype,stretching,spectrumno,Hs,Tp,gamma,coef,seed)

            # Pierson Moscowitz spectrum
            elif spectrum == 'pm':
                spectrumno = 2
                self.argument = 'begin %s ;\n\t\tstretching %d;\n\t\tspectrum %d;'\
                                '\n\t\tpm %.2f %.2f ;\n\t\tcoef %d %d;' \
                                %(wavetype,stretching,spectrumno,Hs,
                                  Tp,coef,seed)

            # check the spreading function
            if spreading is not None:
                self.argument += '\n\t\tspreading 1 %d;'%(spreading)
            # check the embeded stream function
            if embed_sf is not None:
                self.argument += '\n\t\tembed_sf %.2f %d;'%(embed_sf, embed_sf_t0)
            self.argument += '\n\tend;'

        # Stream Wave Input
        if wavetype == 'strf':
            self.waveno = 3
            self.argument = 'begin %s ;\n\t\twave %.2f %.2f 0.0;\n\tend;' \
                            %(wavetype,Hs,Tp)

    def execute(self, filename, folder):
        file_path = os.path.join(folder, filename)
        # check if the hydro input file exists
        if os.path.exists(file_path):
            pass
        else:
            # create directory if non existing
            if not os.path.exists(folder):
                os.makedirs(folder)
            FILE = open(file_path,'w+')
            line1 = 'begin wkin_input ;'
            line2 = 'wavetype %d ;' %self.waveno
            line3 = 'wdepth %.1f ;' %self.wdepth
            line4 = 'end ;'
            file_contents = '%s\n\t%s\n\t%s\n\t%s\n%s\n;\nexit;' \
                            %(line1,line2,line3,self.argument,line4)
            FILE.write(file_contents)
            FILE.close()


if __name__ == '__main__':
    hs = 3
    Tp = 11
    hydro = hydro_input(Hs = hs, Tp = Tp, wdepth = 33,spectrum='jonswap',spreading = 2)
    hydro.execute(filename='sss')



'''
Created on 12/09/2013

@author: mmpe
'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import super
from builtins import range
from future import standard_library
standard_library.install_aliases()

import h5py
import numpy as np
from wetb import gtsdf

import unittest
import os

tmp_path = os.path.dirname(__file__) + "/tmp/"
tfp = os.path.dirname(os.path.abspath(__file__)) + "/test_files/"

class Test_gsdf(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        if not os.path.isdir(tmp_path):
            os.makedirs(tmp_path)

    @classmethod
    def tearDownClass(cls):
        super(Test_gsdf, cls).tearDownClass()
        #shutil.rmtree(tmp_path)

    
    def test_gtsdf_stat(self):
        time, data, info = gtsdf.load(tfp+'test.hdf5')
        fn = tmp_path + "test_stat.hdf5"
        gtsdf.save(fn, data, time=time, **info)
        gtsdf.add_statistic(fn)
        stat_data,info = gtsdf.load_statistic(fn)
        self.assertEqual(data[:,0].min(), stat_data.values[0,0])
        self.assertEqual(stat_data.shape, (49,10))        
        
    def test_gtsdf_compress2stat(self):
        time, data, info = gtsdf.load(tfp+'test.hdf5')
        fn = tmp_path + "test_compress2stat.hdf5"
        gtsdf.save(fn, data, time=time, **info)
        gtsdf.save(tmp_path + "test_compress2stat2.hdf5", data, time=time, dtype=np.float, **info)
        gtsdf.compress2statistics(fn)
        self.assertLess(os.path.getsize(fn)*50, os.path.getsize(tfp+'test.hdf5'))
        
        






if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 20. jul. 2017

@author: mmpe
'''
import numpy as np

from wetb.wind.turbulence.spectra import spectra, spectra_from_time_series
name_format = "mann_l%.1f_ae%.4f_g%.1f_h%d_%dx%dx%d_%.3fx%.2fx%.2f_s%04d%c.turb"

def load(filename, N=(32,32)):
    """Load mann turbulence box
    
    Parameters
    ----------
    filename : str
        Filename of turbulence box
    N : tuple, (ny,nz) or (nx,ny,nz)
        Number of grid points
        
    Returns
    -------
    turbulence_box : nd_array
    
    Examples
    --------
    >>> u = load('turb_u.dat') 
    """
    data = np.fromfile(filename, np.dtype('<f'), -1)
    if len(N)==2:
        ny,nz = N
        nx = len(data)/(ny*nz)
        assert nx==int(nx), "Size of turbulence box (%d) does not match ny x nz (%d), nx=%.2f" % (len(data),ny*nz, nx)
        nx=int(nx)
    else:
        nx,ny,nz = N
        assert len(data) == nx*ny*nz, "Size of turbulence box (%d) does not match nx x ny x nz (%d)" % (len(data),nx*ny*nz)
    return data.reshape(nx , ny * nz)

def load_uvw(filenames, N=(1024,32,32)):
    """Load u, v and w turbulence boxes
    
    Parameters
    ----------
    filenames : list or str
        if list: list of u,v,w filenames
        if str: filename pattern where u,v,w are replaced with '%s'
    N : tuple
        Number of grid point in the x, y and z direction
    
    Returns
    -------
    u,v,w : list of np_array
    
    Examples
    --------
    >>> u,v,w =load_uvw('turb_%s.dat') 
    """
    if isinstance(filenames,str):
        return [load(filenames%uvw, N) for uvw in 'uvw']
    else:
        return [load(f, N) for f in filenames]
    
def parameters2name(no_grid_points,box_dimension,ae23,L, Gamma,high_frq_compensation, seed, folder="./turb/"):
    
    dxyz = tuple(np.array(box_dimension) / no_grid_points)
    return ["./turb/" + name_format % ((L, ae23, Gamma, high_frq_compensation) + no_grid_points + dxyz + (seed, uvw)) for uvw in ['u', 'v', 'w']]    

    
def fit_mann_parameters(spatial_resolution, u, v, w=None, plt=None):
    """Fit mann parameters, ae, L, G
    
    Parameters
    ----------
    spatial_resolution : inf, float or array_like
        Distance between samples in meters
        - For turbulence boxes: 1/dx = Lx/Nx where dx is distance between points, 
        Nx is number of points and Lx is box length in meters
        - For time series: Sample frequency / U
    u : array_like
        The u-wind component\n
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    v : array_like, optional
        The v-wind component
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    w : array_like, optional
        The w-wind component
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    
    Returns
    -------
    ae : int or float
        Alpha epsilon^(2/3) of Mann model
    L : int or float
        Length scale of Mann model
    G : int or float
        Gamma of Mann model
    """
    from wetb.wind.turbulence.mann_parameters import fit_mann_model_spectra
    return fit_mann_model_spectra(*spectra(spatial_resolution, u, v, w), plt=plt)


def fit_mann_parameters_from_time_series(sample_frq, Uvw_lst, plt=None):
    from wetb.wind.turbulence.mann_parameters import fit_mann_model_spectra
    return fit_mann_model_spectra(*spectra_from_time_series(sample_frq, Uvw_lst), plt=plt)


'''
Created on 17/07/2014

@author: MMPE
'''
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from io import open
from builtins import str
from builtins import zip
from future import standard_library
standard_library.install_aliases()
import os
import unittest

from datetime import datetime
from wetb.hawc2.htc_file import HTCFile, HTCLine

import numpy as np


tfp = os.path.join(os.path.dirname(__file__), 'test_files/htcfiles/')  # test file path
class TestHtcFile(unittest.TestCase):

    def test_get_shear(self):
        htc = HTCFile(tfp+'test.htc')
        self.assertEqual(htc.get_shear()(100), 10*(100/119)**.2)
    
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Tue Sep 16 10:21:11 2014

@author: dave
"""
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import
from builtins import str
from future import standard_library
standard_library.install_aliases()

#print(*objects, sep=' ', end='\n', file=sys.stdout)

import os
# import socket
import gc

import numpy as np

import matplotlib.pyplot as plt
import matplotlib as mpl
#from matplotlib.figure import Figure
#from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigCanvas
#from scipy import interpolate as interp
#from scipy.optimize import fmin_slsqp
#from scipy.optimize import minimize
#from scipy.interpolate import interp1d
#import scipy.integrate as integrate
#http://docs.scipy.org/doc/scipy/reference/tutorial/interpolate.html
import pandas as pd

#import openpyxl as px
#import numpy as np

#import windIO
from wetb.prepost import mplutils
from wetb.prepost import Simulations as sim
from wetb.prepost import dlcdefs

plt.rc('font', family='serif')
plt.rc('xtick', labelsize=10)
plt.rc('ytick', labelsize=10)
plt.rc('axes', labelsize=12)
# do not use tex on Gorm and or Jess
#if not socket.gethostname()[:2] in ['g-', 'je', 'j-']:
#    plt.rc('text', usetex=True)
plt.rc('legend', fontsize=11)
plt.rc('legend', numpoints=1)
plt.rc('legend', borderaxespad=0)


def merge_sim_ids(sim_ids, post_dirs, post_dir_save=False, columns=None):
    """
    """

    cols_extra = ['[run_dir]', '[res_dir]', '[wdir]', '[DLC]', '[Case folder]']

    # map the run_dir to the same order as the post_dirs, labels
    run_dirs = []
    # avoid saving merged cases if there is only one!
    if type(sim_ids).__name__ == 'list' and len(sim_ids) == 1:
        sim_ids = sim_ids[0]

    # if sim_id is a list, combine the two dataframes into one
    df_stats = pd.DataFrame()
    if type(sim_ids).__name__ == 'list':
        for ii, sim_id in enumerate(sim_ids):
            if isinstance(post_dirs, list):
                post_dir = post_dirs[ii]
            else:
                post_dir = post_dirs
            cc = sim.Cases(post_dir, sim_id, rem_failed=True)
            df_stats, _, _ = cc.load_stats(leq=False)
            if columns is not None:
                df_stats = df_stats[columns]

            # stats has only a few columns identifying the different cases
            # add some more for selecting them
            dfc = cc.cases2df()
            if '[wsp]' in dfc.columns:
                wsp = '[wsp]'
            else:
                wsp = '[Windspeed]'
            # columns we want to add from cc.cases (cases dict) to stats
            cols_cc = set(cols_extra + [wsp])
            # do not add column twice, some might already be in df stats
            add_cols = list(cols_cc - set(df_stats.columns))
            add_cols.append('[case_id]')
            dfc = dfc[add_cols]
            df_stats = pd.merge(df_stats, dfc, on='[case_id]')
            # FIXME: this is very messy, we can end up with both [wsp] and
            # [Windspeed] columns
            if '[Windspeed]' in df_stats.columns and '[wsp]' in df_stats.columns:
                df_stats.drop('[wsp]', axis=1, inplace=True)
            if wsp != '[Windspeed]':
                df_stats.rename(columns={wsp:'[Windspeed]'}, inplace=True)

            # map the run_dir to the same order as the post_dirs, labels
            run_dirs.append(df_stats['[run_dir]'].unique()[0])

            print('%s Cases loaded.' % sim_id)

            # if specified, save the merged sims elsewhere
            if post_dir_save:
                fpath = os.path.join(post_dir_save, '-'.join(sim_ids) + '.h5')
                try:
                    os.makedirs(post_dir_save)
                except OSError:
                    pass
            else:
                fpath = os.path.join(post_dir, '-'.join(sim_ids) + '.h5')
            fmerged = fpath.replace('.h5', '_statistics.h5')
            if ii == 0:
                # and save somewhere so we can add the second data frame on
                # disc
                df_stats.to_hdf(fmerged, 'table', mode='w', format='table',
                                complevel=9, complib='blosc')
                print('%s merged stats written to: %s' % (sim_id, fpath))
            else:
                # instead of doing a concat in memory, add to the hdf store
                df_stats.to_hdf(fmerged, 'table', mode='r+', format='table',
                                complevel=9, complib='blosc', append=True)
                print('%s merging stats into:      %s' % (sim_id, fpath))
#                df_stats = pd.concat([df_stats, df_stats2], ignore_index=True)
#                df_stats2 = None
            # we might run into memory issues
            del df_stats, _, cc
            gc.collect()
        # and load the reduced combined set
        print('loading merged stats:            %s' % fmerged)
        df_stats = pd.read_hdf(fmerged, 'table')
    else:
        sim_id = sim_ids
        sim_ids = [sim_id]
        post_dir = post_dirs
        if isinstance(post_dirs, list):
            post_dir = post_dirs[0]
        cc = sim.Cases(post_dir, sim_id, rem_failed=True)
        df_stats, _, _ = cc.load_stats(columns=columns, leq=False)
        if columns is not None:
            df_stats = df_stats[columns]
        run_dirs = [df_stats['[run_dir]'].unique()[0]]

        # stats has only a few columns identifying the different cases
        # add some more for selecting them
        dfc = cc.cases2df()
        if '[wsp]' in dfc.columns:
            wsp = '[wsp]'
        else:
            wsp = '[Windspeed]'
        # columns we want to add from cc.cases (cases dict) to stats
        cols_cc = set(cols_extra + [wsp])
        # do not add column twice, some might already be in df stats
        add_cols = list(cols_cc - set(df_stats.columns))
        add_cols.append('[case_id]')
        dfc = dfc[add_cols]
        df_stats = pd.merge(df_stats, dfc, on='[case_id]')
        if '[Windspeed]' in df_stats.columns and '[wsp]' in df_stats.columns:
            df_stats.drop('[wsp]', axis=1, inplace=True)
        if wsp != '[Windspeed]':
            df_stats.rename(columns={wsp:'[Windspeed]'}, inplace=True)

    return run_dirs, df_stats

# =============================================================================
### STAT PLOTS
# =============================================================================

def plot_stats2(sim_ids, post_dirs, plot_chans, fig_dir_base=None, labels=None,
                post_dir_save=False, dlc_ignore=['00'], figsize=(8,6),
                eps=False, ylabels=None, title=True, chans_ms_1hz={}):
    """
    Map which channels have to be compared
    """

    # reduce required memory, only use following columns
    cols = ['[run_dir]', '[DLC]', 'channel', '[res_dir]', '[Windspeed]',
            'mean', 'max', 'min', 'std', '[wdir]', '[Case folder]']

    run_dirs, df_stats = merge_sim_ids(sim_ids, post_dirs,
                                       post_dir_save=post_dir_save)

    plot_dlc_stats(df_stats, plot_chans, fig_dir_base, labels=labels,
                   figsize=figsize, dlc_ignore=dlc_ignore, eps=eps,
                   ylabels=ylabels, title=title, chans_ms_1hz=chans_ms_1hz)


def plot_dlc_stats(df_stats, plot_chans, fig_dir_base, labels=None,
                   figsize=(8,6), dlc_ignore=['00'], run_dirs=None,
                   sim_ids=[], eps=False, ylabels=None, title=True,
                   chans_ms_1hz={}):
    """Create for each DLC an overview plot of the statistics.

    df_stats required columns:
    * [DLC]
    * [run_dir]
    * [wdir]
    * [Windspeed]
    * channel
    * stat parameters

    Parameters
    ----------

    df_stats : pandas.DataFrame

    plot_chans : dict
        Dictionary of channels to be plotted. Key is used for the plot title,
        value is a list of unique channel names that will be included for
        the statistic values. For example,
        plot_chans['$B123 M_x$'] = ['blade1-blade1-node-003-momentvec-x',
        'blade2-blade2-node-003-momentvec-x']

    fig_dir_base : str
        Base directory of where to store all the figures. A new sub-directory
        will be created for each DLC.

    labels : list, default=None
        Labels used in the legend when comparing various DLB's

    figsize : tuple, default=(8,6)

    dlc_ignore : list, default=['dlc00']
        By default all but dlc00 (stair case, wind ramp) are plotted. Add
        more dlc numbers here if necessary.

    run_dirs : list, default=None
        If run_dirs is not defined it will be taken from the unique values
        in the DataFrame. The order of the elements in this list needs to be
        consistent with labels, sim_ids (if defined).

    sim_ids : list, default=[]
        Only used when creating the file name of the figures: appended at
        the end of the file name (which starts with the unique channel name).

    chans_ms_1hz : dict, default={}
        Key/value pairs of channel and list of to be plotten m values. Channel
        refers to plot title/label as used as the key value in plot_chans.

    """

    def fig_epilogue(fig, ax, fname_base):
        ax.grid()
        ax.set_xlim(xlims)
        leg = ax.legend(bbox_to_anchor=(1, 1), loc='lower right', ncol=3)
        leg.get_frame().set_alpha(0.7)
        title_space = 0.0
        if title:
            fig_title = '%s %s' % (dlc_name, ch_dscr)
            # FIXME: dlc_name is assumed to be not in math mode ($$), so
            # escape underscores to avoid latex going bananas
            if mpl.rcParams['text.usetex']:
                fig_title = '%s %s' % (dlc_name.replace('_', '\\_'), ch_dscr)
            fig.suptitle(fig_title)
            title_space = 0.02
        ax.set_xlabel(xlabel)
        if ylabels is not None:
            ax.set_ylabel(ylabels[ch_name])
        fig.tight_layout()
        spacing = 0.94 - title_space - (0.065 * (ii + 1))
        fig.subplots_adjust(top=spacing)
        fig_path = os.path.join(fig_dir, dlc_name)
        if len(sim_ids)==1:
            fname = fname_base + '.png'
        else:
            fname = '%s_%s.png' % (fname_base, '_'.join(sim_ids))
        if not os.path.exists(fig_path):
            os.makedirs(fig_path)
        fig_path = os.path.join(fig_path, fname)
        fig.savefig(fig_path)#.encode('latin-1')
        if eps:
            fig.savefig(fig_path.replace('.png', '.eps'))
        fig.clear()
        print('saved: %s' % fig_path)

    mfcs1 = ['k', 'w']
    mfcs2 = ['b', 'w']
    mfcs3 = ['r', 'w']
    mfcs4 = ['k', 'b']
    mark4 = ['s', 'o', '<', '>']
    mfls4 = ['-', '--']

    required = ['[DLC]', '[run_dir]', '[wdir]', '[Windspeed]', '[res_dir]',
                '[Case folder]']
    cols = df_stats.columns
    for col in required:
        if col not in cols:
            print('plot_dlc_stats requires DataFrame with following columns:')
            print(required)
            print('following column is missing in stats DataFrame:', col)
            return

    if run_dirs is None:
        run_dirs = df_stats['[run_dir]'].unique()

    if not sim_ids:
        sim_ids = []
        for run_dir in run_dirs:
            # in case this is a windows path:
            tmp = run_dir.replace('\\', '/').replace(':', '')
            sim_ids.append(tmp.split('/')[-2])

    # first, take each DLC appart
    for gr_name, gr_dlc in df_stats.groupby(df_stats['[Case folder]']):
        dlc_name = gr_name
        if dlc_name[:3].lower() == 'dlc':
            # FIXME: this is messy since this places a hard coded dependency
            # between [Case folder] and [Case id.] when the tag [DLC] is
            # defined in dlcdefs.py
            dlc_name = gr_name.split('_')[0]
        # do not plot the stats for dlc00
        if dlc_name.lower() in dlc_ignore:
            continue
        # cycle through all the target plot channels
        for ch_dscr, ch_names in plot_chans.items():
            # second, group per channel. Note that when the channel names are not
            # identical, we need to manually pick them.
            # figure file name will be the first channel
            if isinstance(ch_names, list):
                ch_name = ch_names[0]
                fname_base = ch_names[0]#.replace(' ', '_')
                df_chan = gr_dlc[gr_dlc.channel.isin(ch_names)]
            else:
                ch_name = ch_names
                ch_names = [ch_names]
                df_chan = gr_dlc[gr_dlc.channel == ch_names]
                fname_base = ch_names#.replace(' ', '_')

            # if not, than we are missing a channel description, or the channel
            # is simply not available in the given result set
#            if not len(df_chan.channel.unique()) == len(ch_names):
#                continue
            lens = []
            # instead of groupby, select the run_dir in the same order as
            # occuring in the labels and post_dirs lists
            for run_dir in run_dirs:
                lens.append(len(df_chan[df_chan['[run_dir]']==run_dir]))
#            for key, gr_ch_dlc_sid in df_chan.groupby(df_chan['[run_dir]']):
#                lens.append(len(gr_ch_dlc_sid))
            # when the channel is simply not present
            if len(lens) == 0:
                continue
            # when only one of the channels was present, but the set is still
            # complete.
            # FIXME: what if both channels are present?
            if len(ch_names) > 1 and (lens[0] < 1):
                continue
            elif len(ch_names) > 1 and len(lens)==2 and lens[1] < 1:
                continue

            print('start plotting:  %s %s' % (dlc_name.ljust(10), ch_dscr))

            fig, axes = mplutils.make_fig(nrows=1, ncols=1,
                                          figsize=figsize, dpi=120)
            ax = axes[0,0]
            # seperate figure for the mean of the 1Hz equivalent loads
            fig2, axes2 = mplutils.make_fig(nrows=1, ncols=1,
                                            figsize=figsize, dpi=120)
            ax2 = axes2[0,0]

            if fig_dir_base is None and len(sim_ids) < 2:
                res_dir = df_chan['[res_dir]'][:1].values[0]
                fig_dir = os.path.join(fig_dir_base, res_dir)
            elif fig_dir_base is None and len(sim_ids) > 0:
                fig_dir = os.path.join(fig_dir_base, '-'.join(sim_ids))
#            elif fig_dir_base and len(sim_ids) < 2:
#                res_dir = df_chan['[res_dir]'][:1].values[0]
#                fig_dir = os.path.join(fig_dir_base, res_dir)
            elif fig_dir_base is not None:
                # create the compare directory if not defined
                fig_dir = fig_dir_base

            # if we have a list of different cases, we also need to group those
            # because the sim_id wasn't saved before in the data frame,
            # we need to derive that from the run dir
            # if there is only one run dir nothing changes
#            sid_names = []
            # for clarity, set off-set on wind speed when comparing two DLB's
            if len(lens)==2:
                windoffset = [-0.2, 0.2]
                dirroffset = [-5, 5]
            else:
                windoffset = [0]
                dirroffset = [0]
            # in case of a fully empty plot xlims will remain None and there
            # is no need to save the plot
            xlims = None
            # instead of groupby, select the run_dir in the same order as
            # occuring in the labels, post_dirs lists
            for ii, run_dir in enumerate(run_dirs):
                gr_ch_dlc_sid = df_chan[df_chan['[run_dir]']==run_dir]
                if len(gr_ch_dlc_sid) < 1:
                    print('no data for run_dir:', run_dir)
                    continue
#            for run_dir, gr_ch_dlc_sid in df_chan.groupby(df_chan['[run_dir]']):
                if labels is None:
                    sid_name = sim_ids[ii]
                else:
                    sid_name = labels[ii]
#                sid_names.append(sid_name)
                print('   sim_id/label:', sid_name)
                # FIXME: will this go wrong in PY3?
                if dlc_name.lower() in ['dlc61', 'dlc62']:
                    key = '[wdir]'
                    xdata = gr_ch_dlc_sid[key].values + dirroffset[ii]
                    xlabel = 'wind direction [deg]'
                    xlims = [0, 360]
                else:
                    key = '[Windspeed]'
                    xdata = gr_ch_dlc_sid[key].values + windoffset[ii]
                    xlabel = 'Wind speed [m/s]'
                    xlims = [3, 27]
                dmin = gr_ch_dlc_sid['min'].values
                dmean = gr_ch_dlc_sid['mean'].values
                dmax = gr_ch_dlc_sid['max'].values
                dstd = gr_ch_dlc_sid['std'].values
                if len(sim_ids)==1:
                    lab1 = 'mean'
                    lab2 = 'min'
                    lab3 = 'max'
                    lab4 = '1Hz EqL'
                else:
                    lab1 = 'mean %s' % sid_name
                    lab2 = 'min %s' % sid_name
                    lab3 = 'max %s' % sid_name
                    lab4 = '1Hz EqL %s' % sid_name
                mfc1 = mfcs1[ii]
                mfc2 = mfcs2[ii]
                mfc3 = mfcs3[ii]
                ax.errorbar(xdata, dmean, mec='k', marker='o', mfc=mfc1, ls='',
                            label=lab1, alpha=0.7, yerr=dstd, ecolor='k')
                ax.plot(xdata, dmin, mec='b', marker='^', mfc=mfc2, ls='',
                        label=lab2, alpha=0.7)
                ax.plot(xdata, dmax, mec='r', marker='v', mfc=mfc3, ls='',
                        label=lab3, alpha=0.7)

                # mean of 1Hz equivalent loads
                ms = []
                if ch_dscr in chans_ms_1hz:
                    ms = chans_ms_1hz[ch_dscr]
                for im, m in enumerate(ms):
                    # average over seed and possibly yaw angles
                    # wind speed or yaw inflow according to dlc case
                    gr_key = gr_ch_dlc_sid[key]
                    d1hz = gr_ch_dlc_sid[m].groupby(gr_key).mean()

                    ax2.plot(d1hz.index, d1hz.values, mec=mfcs4[ii], alpha=0.7,
                             marker=mark4[im], ls=mfls4[ii], mfc=mfc1,
                             label=lab4, color=mfcs4[ii])

#            for wind, gr_wind in  gr_ch_dlc.groupby(df_stats['[Windspeed]']):
#                wind = gr_wind['[Windspeed]'].values
#                dmin = gr_wind['min'].values#.mean()
#                dmean = gr_wind['mean'].values#.mean()
#                dmax = gr_wind['max'].values#.mean()
##                dstd = gr_wind['std'].mean()
#                ax.plot(wind, dmean, 'ko', label='mean', alpha=0.7)
#                ax.plot(wind, dmin, 'b^', label='min', alpha=0.7)
#                ax.plot(wind, dmax, 'rv', label='max', alpha=0.7)
##                ax.errorbar(wind, dmean, c='k', ls='', marker='s', mfc='w',
##                        label='mean and std', yerr=dstd)
#            if str(dlc_name) not in ['61', '62']:
#                ax.set_xticks(gr_ch_dlc_sid['[Windspeed]'].values)

            # don't save empyt plots
            if xlims is None:
                continue

            fig_epilogue(fig, ax, fname_base)

            # don't save empty plots
            if len(ms) < 1:
                continue

            fig_epilogue(fig2, ax2, fname_base + '_1hz_eql')


class PlotStats(object):

    def __init__(self):
        pass

    def load_stats(self, sim_ids, post_dirs, post_dir_save=False):

        self.sim_ids = sim_ids
        self.post_dirs = post_dirs

        # reduce required memory, only use following columns
        cols = ['[run_dir]', '[DLC]', 'channel', '[res_dir]', '[Windspeed]',
                'mean', 'max', 'min', 'std', '[wdir]']

        # if sim_id is a list, combine the two dataframes into one
        df_stats = pd.DataFrame()
        if type(sim_ids).__name__ == 'list':
            for ii, sim_id in enumerate(sim_ids):
                if isinstance(post_dirs, list):
                    post_dir = post_dirs[ii]
                else:
                    post_dir = post_dirs
                cc = sim.Cases(post_dir, sim_id, rem_failed=True)
                df_stats, _, _ = cc.load_stats(columns=cols, leq=False)
                print('%s Cases loaded.' % sim_id)

                # if specified, save the merged sims elsewhere
                if post_dir_save:
                    fpath = os.path.join(post_dir_save, '-'.join(sim_ids) + '.h5')
                    try:
                        os.makedirs(post_dir_save)
                    except OSError:
                        pass
                else:
                    fpath = os.path.join(post_dir, '-'.join(sim_ids) + '.h5')
                if ii == 0:
                    # and save somewhere so we can add the second data frame on
                    # disc
                    df_stats.to_hdf(fpath, 'table', mode='w', format='table',
                                    complevel=9, complib='blosc')
                    print('%s merged stats written to: %s' % (sim_id, fpath))
                else:
                    # instead of doing a concat in memory, add to the hdf store
                    df_stats.to_hdf(fpath, 'table', mode='r+', format='table',
                                    complevel=9, complib='blosc', append=True)
                    print('%s merging stats into:      %s' % (sim_id, fpath))
                # we might run into memory issues
                del df_stats, _, cc
                gc.collect()
            # and load the reduced combined set
            print('loading merged stats:            %s' % fpath)
            df_stats = pd.read_hdf(fpath, 'table')
        else:
            sim_id = sim_ids
            sim_ids = [sim_id]
            post_dir = post_dirs
            cc = sim.Cases(post_dir, sim_id, rem_failed=True)
            df_stats, _, _ = cc.load_stats(leq=False)

        return df_stats

    def select_extremes_blade_radial(self, df):
        """
        For each radial position of the blade, find the extremes
        """

        def selector(x):
            """
            select following channels:
            'local-blade%1i-node-%03i-momentvec-x'
            'blade2-blade2-node-003-momentvec-y'
            """
            if x[:11] == 'local-blade' and x[22:31] == 'momentvec':
                return True
            else:
                return False

        # find all blade local load channels
        criteria = df.channel.map(lambda x: selector(x))
        df = df[criteria]
        # split channel columns so we can select channels properly
        df = df.join(df.channel.apply(lambda x: pd.Series(x.split('-'))))

        df_ext = {'dlc':[], 'case':[], 'node':[], 'max':[], 'min':[], 'comp':[]}

        def fillvalues(x, ii, maxmin):
            x['node'].append(m_group[3].ix[ii])
            x['dlc'].append(m_group['[DLC]'].ix[ii])
            x['case'].append(m_group['[case_id]'].ix[ii])
            x['comp'].append(m_group[5].ix[ii])
            if maxmin == 'max':
                x['max'].append(m_group['max'].ix[ii])
                x['min'].append(np.nan)
            else:
                x['max'].append(np.nan)
                x['min'].append(m_group['min'].ix[ii])
            return x

        # we take blade1, blade2, and blade3
        df_b2 = df[df[0]=='local']
#        df_b2 = df_b2[df_b2[1]=='blade2']
        df_b2 = df_b2[df_b2[4]=='momentvec']
#        df_b2 = df_b2[df_b2[5]=='x']
        # make sure we only have blade1, 2 and 3
        assert set(df_b2[1].unique()) == set(['blade3', 'blade2', 'blade1'])
#        # number of nodes
#        nrnodes = len(df_b2[3].unique())
        # group by node number, and take the max
        for nodenr, group in df_b2.groupby(df_b2[3]):
            print(nodenr, end='   ')
            for comp, m_group in df_b2.groupby(group[5]):
                print(comp)
                imax = m_group['max'].argmax()
                imin = m_group['min'].argmin()
                df_ext = fillvalues(df_ext, imax, 'max')
                df_ext = fillvalues(df_ext, imin, 'min')

        df_ext = pd.DataFrame(df_ext)
        df_ext.sort(columns='node', inplace=True)

        return df_ext

    def plot_extremes_blade_radial(self, df_ext, fpath):
        nrows = 2
        ncols = 2
        figsize = (11,7.15)
        fig, axes = mplutils.make_fig(nrows=nrows, ncols=ncols, figsize=figsize)

#        self.col = ['r', 'k']
#        self.alf = [1.0, 0.7]
#        self.i = 0

        mx_max = df_ext['max'][df_ext.comp == 'x'].dropna()
        mx_min = df_ext['min'][df_ext.comp == 'x'].dropna()
        my_max = df_ext['max'][df_ext.comp == 'y'].dropna()
        my_min = df_ext['min'][df_ext.comp == 'y'].dropna()
#        nodes = df_ext.node.ix[mx_max.index]
        axes[0,0].plot(mx_max, 'r^', label='$M_{x_{max}}$')
        axes[0,1].plot(mx_min, 'bv', label='$M_{x_{min}}$')
        axes[1,0].plot(my_max, 'r^', label='$M_{y_{max}}$')
        axes[1,1].plot(my_min, 'bv', label='$M_{y_{min}}$')

        axs = axes.ravel()
        for ax in axs:
            ax.grid()
            ax.legend(loc='best')

#        axs[0].set_xticklabels([])
#        axs[1].set_xticklabels([])
#        axs[2].set_xticklabels([])
#        axs[-1].set_xlabel('time [s]')

        fig.tight_layout()
        fig.subplots_adjust(hspace=0.06)
        fig.subplots_adjust(top=0.98)

        fdir = os.path.dirname(fpath)
#        fname = os.path.basename(fpath)

        if not os.path.exists(fdir):
            os.makedirs(fdir)
        print('saving: %s ...' % fpath, end='')
        fig.savefig(fpath)#.encode('latin-1')
        print('done')
        fig.clear()

        # save as tables
        df_ext.ix[mx_max.index].to_excel(fpath.replace('.png', '_mx_max.xls'))
        df_ext.ix[mx_min.index].to_excel(fpath.replace('.png', '_mx_min.xls'))
        df_ext.ix[my_max.index].to_excel(fpath.replace('.png', '_my_max.xls'))
        df_ext.ix[my_min.index].to_excel(fpath.replace('.png', '_my_min.xls'))

    def extract_leq_blade_radial(self, df_leq, fpath):

        def selector(x):
            """
            select following channels:
            'local-blade%1i-node-%03i-momentvec-x'
            'blade2-blade2-node-003-momentvec-y'
            """
            if x[:11] == 'local-blade' and x[22:31] == 'momentvec':
                return True
            else:
                return False

        # find all blade local load channels
        criteria = df_leq.channel.map(lambda x: selector(x))
        df = df_leq[criteria]
        # split channel columns so we can select channels properly
        df = df.join(df.channel.apply(lambda x: pd.Series(x.split('-'))))
        df.sort(columns=3, inplace=True)
        assert set(df[1].unique()) == set(['blade3', 'blade2', 'blade1'])

        leqs = list(df.keys())[1:10]
        df_ext = {leq:[] for leq in leqs}
        df_ext['node'] = []
        df_ext['comp'] = []

        for nodenr, group in df.groupby(df[3]):
            print(nodenr, end='   ')
            for comp, m_group in df.groupby(group[5]):
                print(comp)
                for leq in leqs:
                    df_ext[leq].append(m_group[leq].max())
                df_ext['node'].append(nodenr)
                df_ext['comp'].append(comp)

        df_ext = pd.DataFrame(df_ext)
        df_ext.sort(columns='node', inplace=True)

        df_ext[df_ext.comp=='x'].to_excel(fpath.replace('.xls', '_x.xls'))
        df_ext[df_ext.comp=='y'].to_excel(fpath.replace('.xls', '_y.xls'))
        df_ext[df_ext.comp=='z'].to_excel(fpath.replace('.xls', '_z.xls'))

        return df_ext


class PlotPerf(object):

    def __init__(self, nrows=4, ncols=1, figsize=(14,11)):

        self.fig, self.axes = mplutils.make_fig(nrows=nrows, ncols=ncols,
                                                 figsize=figsize)
#        self.axs = self.axes.ravel()
        self.col = ['r', 'k']
        self.alf = [1.0, 0.7]
        self.i = 0

    def plot(self, res, label_id):
        """
        """
        i = self.i

        sim_id = label_id
        time = res.sig[:,0]
        self.t0, self.t1 = time[0], time[-1]

        # find the wind speed
        for channame, chan in res.ch_dict.items():
            if channame.startswith('windspeed-global-Vy-0.00-0.00'):
                break
        wind = res.sig[:,chan['chi']]

        chi = res.ch_dict['bearing-shaft_rot-angle_speed-rpm']['chi']
        rpm = res.sig[:,chi]

        chi = res.ch_dict['bearing-pitch1-angle-deg']['chi']
        pitch = res.sig[:,chi]

        chi = res.ch_dict['tower-tower-node-001-momentvec-x']['chi']
        tx = res.sig[:,chi]

        chi = res.ch_dict['tower-tower-node-001-momentvec-y']['chi']
        ty = res.sig[:,chi]

        chi = res.ch_dict['DLL-2-inpvec-2']['chi']
        power = res.sig[:,chi]

        try:
            chi = res.ch_dict['Tors_e-1-100.11']['chi']
        except KeyError:
            chi = res.ch_dict['Tors_e-1-86.50']['chi']
        tors_1 = res.sig[:,chi]

#        try:
#            chi = res.ch_dict['Tors_e-1-96.22']['chi']
#        except:
#            chi = res.ch_dict['Tors_e-1-83.13']['chi']
#        tors_2 = res.sig[:,chi]

        try:
            chi = res.ch_dict['Tors_e-1-84.53']['chi']
        except:
            chi = res.ch_dict['Tors_e-1-73.21']['chi']
        tors_3 = res.sig[:,chi]

        ax = self.axes.ravel()
        ax[0].plot(time, wind, self.col[i]+'--', label='%s wind speed' % sim_id,
                   alpha=self.alf[i])
        ax[0].plot(time, pitch, self.col[i]+'-.', label='%s pitch' % sim_id,
                   alpha=self.alf[i])
        ax[0].plot(time, rpm, self.col[i]+'-', label='%s RPM' % sim_id,
                   alpha=self.alf[i])

        ax[1].plot(time, tx, self.col[i]+'--', label='%s Tower FA' % sim_id,
                   alpha=self.alf[i])
        ax[1].plot(time, ty, self.col[i]+'-', label='%s Tower SS' % sim_id,
                   alpha=self.alf[i])

        ax[2].plot(time, power/1e6, self.col[i]+'-', alpha=self.alf[i],
                   label='%s El Power' % sim_id)

        ax[3].plot(time, tors_1, self.col[i]+'--', label='%s torsion tip' % sim_id,
                   alpha=self.alf[i])
#        ax[3].plot(time, tors_2, self.col[i]+'-.', label='%s torsion 96 pc' % sim_id,
#                   alpha=self.alf[i])
#        ax[3].plot(time, tors_3, self.col[i]+'-', label='%s torsion 84 pc' % sim_id,
#                   alpha=self.alf[i])

        self.i += 1

    def final(self, fig_path, fig_name):

        axs = self.axes.ravel()

        for ax in axs:
            ax.set_xlim([self.t0, self.t1])
            ax.grid()
            ax.legend(loc='best')

        axs[0].set_xticklabels([])
        axs[1].set_xticklabels([])
        axs[2].set_xticklabels([])
        axs[-1].set_xlabel('time [s]')

        self.fig.tight_layout()
        self.fig.subplots_adjust(hspace=0.06)
        self.fig.subplots_adjust(top=0.98)

        if not os.path.exists(fig_path):
            os.makedirs(fig_path)
        fname = os.path.join(fig_path, fig_name)
        print('saving: %s ...' % fname, end='')
        self.fig.savefig(fname)#.encode('latin-1')
        print('done')
        self.fig.clear()


def plot_dlc01_powercurve(sim_ids, post_dirs, run_dirs, fig_dir_base):
    """
    Create power curve based on steady DLC01 results
    Use the same format as for HS2 for easy comparison!
    """



def plot_dlc00(sim_ids, post_dirs, run_dirs, fig_dir_base=None, labels=None,
               cnames=['dlc00_stair_wsp04_25_noturb.htc',
                       'dlc00_ramp_wsp04_25_04_noturb.htc'], figsize=(14,11)):
    """
    This version is an update over plot_staircase.
    """

    stairs = []
    # if sim_id is a list, combine the two dataframes into one
    if type(sim_ids).__name__ == 'list':
        for ii, sim_id in enumerate(sim_ids):
            if isinstance(post_dirs, list):
                post_dir = post_dirs[ii]
            else:
                post_dir = post_dirs
            stairs.append(sim.Cases(post_dir, sim_id, rem_failed=True))
    else:
        post_dir = post_dirs
        stairs.append(sim.Cases(post_dir, sim_id, rem_failed=True))

    for cname in cnames:
        fp = PlotPerf(figsize=figsize)
        for i, cc in enumerate(stairs):
            if isinstance(cname, list):
                _cname = cname[i]
            else:
                _cname = cname
            if _cname in cc.cases_fail:
                print('no result for %s' % cc.sim_id)
                continue
            cc.change_results_dir(run_dirs[i])
            try:
                res = cc.load_result_file(cc.cases[_cname])
            except KeyError:
                for k in sorted(cc.cases.keys()):
                    print(k)
                print('-'*79)
                print(cc.sim_id, _cname)
                print('-'*79)
                raise KeyError
            if labels is not None:
                label = labels[i]
            else:
                label = cc.sim_id
            fp.plot(res, label)
        dlcf = 'dlc' + cc.cases[_cname]['[DLC]']
        fig_path = os.path.join(fig_dir_base, dlcf)
        fp.final(fig_path, _cname.replace('.htc', '.png'))


def plot_staircase(sim_ids, post_dirs, run_dirs, fig_dir_base=None,
                   cname='dlc00_stair_wsp04_25_noturb.htc'):
    """
    Default stair and ramp names:

    dlc00_stair_wsp04_25_noturb
    dlc00_ramp_wsp04_25_04_noturb
    """

    stairs = []

    col = ['r', 'k']
    alf = [1.0, 0.7]

    # if sim_id is a list, combine the two dataframes into one
    if type(sim_ids).__name__ == 'list':
        for ii, sim_id in enumerate(sim_ids):
            if isinstance(post_dirs, list):
                post_dir = post_dirs[ii]
            else:
                post_dir = post_dirs
            stairs.append(sim.Cases(post_dir, sim_id, rem_failed=True))
    else:
        sim_id = sim_ids
        sim_ids = [sim_id]
        post_dir = post_dirs
        stairs.append(sim.Cases(post_dir, sim_id, rem_failed=True))

    fig, axes = mplutils.make_fig(nrows=3, ncols=1, figsize=(14,10))
    ax = axes.ravel()

    for i, cc in enumerate(stairs):
        if cname in cc.cases_fail:
            print('no result for %s' % cc.sim_id)
            continue
        cc.change_results_dir(run_dirs[i])
        res = cc.load_result_file(cc.cases[cname])
        respath = cc.cases[cname]['[run_dir]']
        fname = os.path.join(respath, cname)
        df_respost = pd.read_hdf(fname + '_postres.h5', 'table')
        sim_id = cc.sim_id
        time = res.sig[:,0]
        t0, t1 = time[0], time[-1]

        # find the wind speed
        for channame, chan in res.ch_dict.items():
            if channame.startswith('windspeed-global-Vy-0.00-0.00'):
                break
        wind = res.sig[:,chan['chi']]
        chi = res.ch_dict['bearing-pitch1-angle-deg']['chi']
        pitch = res.sig[:,chi]

        chi = res.ch_dict['bearing-shaft_rot-angle_speed-rpm']['chi']
        rpm = res.sig[:,chi]

        chi = res.ch_dict['bearing-pitch1-angle-deg']['chi']
        pitch = res.sig[:,chi]

        chi = res.ch_dict['tower-tower-node-001-momentvec-x']['chi']
        tx = res.sig[:,chi]

        chi = res.ch_dict['tower-tower-node-001-momentvec-y']['chi']
        ty = res.sig[:,chi]

        chi = res.ch_dict['DLL-2-inpvec-2']['chi']
        power = res.sig[:,chi]

        chi = res.ch_dict['DLL-2-inpvec-2']['chi']
        power_mech = df_respost['stats-shaft-power']

        ax[0].plot(time, wind, col[i]+'--', label='%s wind speed' % sim_id,
                   alpha=alf[i])
        ax[0].plot(time, pitch, col[i]+'-.', label='%s pitch' % sim_id,
                   alpha=alf[i])
        ax[0].plot(time, rpm, col[i]+'-', label='%s RPM' % sim_id,
                   alpha=alf[i])

        ax[1].plot(time, tx, col[i]+'--', label='%s Tower FA' % sim_id,
                   alpha=alf[i])
        ax[1].plot(time, ty, col[i]+'-', label='%s Tower SS' % sim_id,
                   alpha=alf[i])

        ax[2].plot(time, power/1e6, col[i]+'-', label='%s El Power' % sim_id,
                   alpha=alf[i])
        ax[2].plot(time, power_mech/1e3, col[i]+'-', alpha=alf[i],
                   label='%s Mech Power' % sim_id)

    ax[0].set_xlim([t0, t1])
    ax[0].grid()
    ax[0].legend(loc='best')
    ax[0].set_xticklabels([])
#    ax[0].set_xlabel('time [s]')

    ax[1].set_xlim([t0, t1])
    ax[1].grid()
    ax[1].legend(loc='best')
    ax[1].set_xticklabels([])
#    ax[1].set_xlabel('time [s]')

    ax[2].set_xlim([t0, t1])
    ax[2].grid()
    ax[2].legend(loc='best')
    ax[2].set_xlabel('time [s]')

    fig.tight_layout()
    fig.subplots_adjust(hspace=0.06)
    fig.subplots_adjust(top=0.92)

    if not os.path.exists(fig_dir_base):
        os.makedirs(fig_dir_base)
    fig_path = os.path.join(fig_dir_base, '-'.join(sim_ids) + '_stair.png')
    print('saving: %s ...' % fig_path, end='')
    fig.savefig(fig_path)#.encode('latin-1')
    print('done')
    fig.clear()


if __name__ == '__main__':

    # auto configure directories: assume you are running in the root of the
    # relevant HAWC2 model
    # and assume we are in a simulation case of a certain turbine/project
    P_RUN, P_SOURCE, PROJECT, sim_id, P_MASTERFILE, MASTERFILE, POST_DIR \
        = dlcdefs.configure_dirs()

    # -------------------------------------------------------------------------
#    # manually configure all the dirs
#    p_root_remote = '/mnt/hawc2sim'
#    p_root_local = '/home/dave/DTU/Projects/AVATAR/'
#    # project name, sim_id, master file name
#    PROJECT = 'DTU10MW'
#    sim_id = 'C0014'
#    MASTERFILE = 'dtu10mw_master_C0014.htc'
#    # MODEL SOURCES, exchanche file sources
#    P_RUN = os.path.join(p_root_remote, PROJECT, sim_id+'/')
#    P_SOURCE = os.path.join(p_root_local, PROJECT)
#    # location of the master file
#    P_MASTERFILE = os.path.join(p_root_local, PROJECT, 'htc', '_master/')
#    # location of the pre and post processing data
#    POST_DIR = os.path.join(p_root_remote, PROJECT, 'python-prepost-data/')
#    force_dir = P_RUN
    # -------------------------------------------------------------------------

    # PLOT STATS, when comparing cases
    sim_ids = [sim_id]
    run_dirs = [P_RUN]
    figdir = os.path.join(P_RUN, '..', 'dlcplots/%s' % sim_id)

    print('='*79)
    print('   P_RUN: %s' % P_RUN)
    print('P_SOURCE: %s' % P_SOURCE)
    print(' PROJECT: %s' % PROJECT)
    print('  sim_id: %s' % sim_id)
    print('  master: %s' % MASTERFILE)
    print('  figdir: %s' % figdir)
    print('='*79)

    plot_stats2(sim_ids, POST_DIR, fig_dir_base=figdir)
    plot_dlc00(sim_ids, POST_DIR, run_dirs, fig_dir_base=figdir)



import os
import inspect
from wetb.utils.cluster_tools.os_path import cluster_path, pjoin


class Template():

    def __init__(self, template):
        self.template = template

    def __call__(self, **kwargs):
        s = self.template
        found = True
        while found:
            found = False
            for k, v in dict(kwargs).items():
                if "[%s]" % k in s:
                    found = True
                s = s.replace("[%s]" % k, str(v))
        return s


pbs_template = Template('''### Jobid
#PBS -N [jobname]
### Standard Output
#PBS -o [stdout_filename]
[stderr]
#PBS -W umask=0003
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=[walltime]
#PBS -l nodes=[nodes]:ppn=[ppn]
### Queue name
#PBS -q [queue]
cd [workdir]
mkdir -p stdout
if [ -z "$PBS_JOBID" ]; then echo "Run using qsub"; exit ; fi
pwd
[commands]
exit
''')


class PBSFile():
    _walltime = "00:30:00"

    def __init__(self, workdir, jobname, commands, queue='workq', walltime='00:10:00', nodes=1, ppn=1, merge_std=True):
        """Description

        Parameters
        ----------
        walltime : int, str
            wall time as string ("hh:mm:ss") or second (integer)

        """
        self.workdir = workdir
        self.jobname = jobname
        self.commands = commands
        self.queue = queue
        self.walltime = walltime
        self.nodes = nodes
        self.ppn = ppn
        self.merge_std = merge_std
        self.stdout_filename = pjoin(workdir, './stdout/%s.out' % jobname)
        self.filename = "pbs_in/%s.in" % self.jobname

    @property
    def walltime(self):
        return self._walltime

    @walltime.setter
    def walltime(self, walltime):
        if isinstance(walltime, (float, int)):
            from math import ceil
            h = walltime // 3600
            m = (walltime % 3600) // 60
            s = ceil(walltime % 60)
            self._walltime = "%02d:%02d:%02d" % (h, m, s)
        else:
            self._walltime = walltime

    def __str__(self):
        if self.merge_std:
            stderr = "### merge stderr into stdout\n#PBS -j oe"
        else:
            stderr = "### Standard Error\n#PBS -e ./err/[jobname].err"
        if callable(self.commands):
            commands = self.commands()
        else:
            commands = self.commands
        return pbs_template(workdir=cluster_path(self.workdir),
                            stdout_filename=cluster_path(self.stdout_filename),
                            stderr=stderr,
                            jobname=self.jobname,
                            queue=self.queue,
                            walltime=self.walltime,
                            nodes=self.nodes,
                            ppn=self.ppn,
                            commands=commands)

    def save(self, model_path=None, filename=None):
        model_path = model_path or self.workdir
        self.filename = filename or self.filename
        filename = os.path.join(model_path, self.filename)
        os.makedirs(os.path.dirname(filename), exist_ok=True)
        with open(filename, 'w', newline='\n') as fid:
            fid.write(str(self))


multirunner_template = Template("""echo "[make_dict]
" | python

for node in `cat $PBS_NODEFILE | sort | uniq`
do
     
     ssh -T $node << EOF &
cd [workdir]
python -c "[start_jobs]
"
EOF
done
wait
""")


class PBSMultiRunner(PBSFile):
    def __init__(self, workdir, queue='workq', walltime='01:00:00', nodes=1, ppn=1, merge_std=True):
        commands = multirunner_template(make_dict=self.get_src(self.make_dict),
                                        start_jobs=self.get_src(self.start_jobs),
                                        workdir=cluster_path(workdir)).replace("self.ppn", str(ppn))
        jobname = "%s" % os.path.basename(os.path.abspath(workdir))
        jobname = 'pbs_multirunner'
        PBSFile.__init__(self, workdir, jobname, commands, queue, walltime=walltime,
                         nodes=nodes, ppn=ppn, merge_std=merge_std)
        self.filename = "%s.all" % self.jobname

    def make_dict(self):
        import os
        import glob
        import numpy as np
        import re

        # find available nodes
        with open(os.environ['PBS_NODEFILE']) as fid:
            files = set([f.strip() for f in fid.readlines() if f.strip() != ''])
        pbs_files = [os.path.join(root, f) for root, folders, f_lst in os.walk('.') for f in f_lst if f.endswith('.in')]

        # Make a list of [(pbs_in_filename, stdout_filename, walltime),...]
        pat = re.compile(r'[\s\S]*#\s*PBS\s+-o\s+(.*)[\s\S]*(\d\d:\d\d:\d\d)[\s\S]*')

        def get_info(f):
            try:
                with open(f) as fid:
                    return (f,) + pat.match(fid.read()).groups()
            except Exception:
                return (f, f.replace('.in', '.out'), '00:30:00')
        pbs_info_lst = map(get_info, pbs_files)

        # sort wrt walltime
        pbs_info_lst = sorted(pbs_info_lst, key=lambda fow: tuple(map(int, fow[2].split(':'))))[::-1]
        # make dict {node1: pbs_info_lst1, ...} and save
        d = dict([(f, pbs_info_lst[i::len(files)]) for i, f in enumerate(files)])
        with open('pbs.dict', 'w') as fid:
            fid.write(str(d))

    def start_jobs(self):
        import os
        import multiprocessing
        import platform
        import time
        with open('pbs.dict') as fid:
            pbs_info_lst = eval(fid.read())[platform.node()]
        arg_lst = ['echo starting %s && mkdir -p %s && env PBS_JOBID=$PBS_JOBID %s &> %s && echo finished %s' %
                   (f, os.path.dirname(o), f, o, f) for f, o, _ in pbs_info_lst]
        print(arg_lst[0])
        print('Starting %d jobs on %s' % (len(arg_lst), platform.node()))
        pool = multiprocessing.Pool(int('$PBS_NUM_PPN'))
        res = pool.map_async(os.system, arg_lst)
        t = time.time()
        for (f, _, _), r in zip(pbs_info_lst, res.get()):
            print('%-50s\t%s' % (f, ('Errorcode %d' % r, 'Done')[r == 0]))
        print('Done %d jobs on %s in %ds' % (len(arg_lst), platform.node(), time.time() - t))

    def get_src(self, func):
        src_lines = inspect.getsource(func).split("\n")[1:]
        indent = len(src_lines[0]) - len(src_lines[0].lstrip())
        return "\n".join([l[indent:] for l in src_lines])


if __name__ == '__main__':
    pbsmr = PBSMultiRunner(workdir="W:/simple1", queue='workq', nodes=2, ppn=10)
    pbsmr.save("w:/simple1")



'''
Created on 29. mar. 2017

@author: mmpe
'''
import unittest
import numpy as np
from wetb.signal.filters._differentiation import differentiation

class Test(unittest.TestCase):


    def testDifferentiation(self):
        np.testing.assert_array_equal(differentiation([1,2,1,0,1,1]), [1,0,-1,0,.5,0])
        np.testing.assert_array_equal(differentiation([1,2,1,0,1,1], 'left'), [np.nan, 1,-1,-1,1,0])
        np.testing.assert_array_equal(differentiation([1,2,1,0,1,1], 'right'), [1,-1,-1,1,0, np.nan])
        


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testDifferentiation']
    unittest.main()


'''
Created on 27/11/2015

@author: MMPE
'''


import numpy as np
import warnings
def spectrum(x, y=None, k=1):
    """PSD or Cross spectrum (only positive half)

    If input time series are two dimensional, then columns are interpreted
    as different time series and the mean spectrum is returned

    Parameters
    ----------
    x : array_like
        Time series
    y : array_like, optional
        If array_like, cross spectrum of x and y is returned
        if None, cross spectrum of x and x (i.e. PSD) is returned
    k : int or float, optional
        Max wave number
    """
    if x is None:
        return None
    def fft(x):
        return np.fft.fft(x.T).T / len(x)

    if y is None or x is y:
        fftx = fft(x)
        fftx = fftx * np.conj(fftx)  # PSD, ~ fft(x)**2
    else:
        fftx = fft(x) * np.conj(fft(y))  # Cross spectra

    fftx = fftx[:len(fftx) // 2 ] * 2  # positive half * 2

#    if len(fftx.shape) == 2:
#        fftx = np.mean(fftx, 1)
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")  
        return np.real(fftx * len(x) / (2 * k))[1:]

def spectra(spatial_resolution, u, v=None, w=None, detrend=True):
    """Return the wave number, the uu, vv, ww autospectra and the uw cross spectra

    Parameters
    ----------
    spatial_resolution : int, float or array_like
        Distance between samples in meters
        - For turbulence boxes: 1/dx = Nx/Lx where dx is distance between points, 
        Nx is number of points and Lx is box length in meters
        - For time series: Sample frequency / U
    u : array_like
        The u-wind component\n
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    v : array_like, optional
        The v-wind component
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    w : array_like, optional
        The w-wind component
        - if shape is (r,): One time series with *r* observations\n
        - if shape is (r,c): *c* different time series with *r* observations\n
    detrend : boolean, optional
        if True (default) wind speeds are detrended before calculating spectra

    Returns
    -------
    k1, uu[, vv[, ww, uw]] : 2-5 x array_like
        - k1: wave number
        - uu: The uu auto spectrum\n
        - vv: The vv auto spectrum, only if v is provided\n
        - ww: The ww auto spectrum, only if w is provided\n
        - uw: The uw cross spectrum, only if w is provided\n

        For two dimensional input, the mean spectra are returned
    """
    assert isinstance(spatial_resolution, (int, float))

    k = 2 * np.pi * spatial_resolution
    if v is not None: assert u.shape==v.shape
    if w is not None: assert u.shape==w.shape
    
    if 1 and len(u.shape)==2:
        assert np.abs(np.mean(u,0)).max()<1
        if v is not None: assert np.abs(np.mean(v,0)).max()<1
        if w is not None: assert np.abs(np.mean(w,0)).max()<1
        if isinstance(k, float):
            k = np.repeat(k,u.shape[1])
        else:
            assert u.shape[1]==k.shape[0]
        k1_vec = np.array([np.linspace(0, k_ / 2, len(u) / 2)[1:] for k_ in k]).T
    else:
        assert np.abs(np.mean(u))<1
        if v is not None: assert np.abs(np.mean(v))<1
        if w is not None: assert np.abs(np.mean(w))<1
        assert isinstance(k,float)
        k1_vec = np.linspace(0, k / 2, len(u) / 2)[1:]
    if detrend:
        u, v, w = detrend_wsp(u, v, w)

    return [k1_vec] + [spectrum(x1, x2, k=k) for x1, x2 in [(u, u), (v, v), (w, w), (w, u)]]


def spectra_from_time_series(sample_frq, Uvw_lst):
    """Return the wave number, the uu, vv, ww autospectra and the uw cross spectra

    Parameters
    ----------
    sample_frq : int, float or array_like
        Sample frequency
    Uvw_lst : array_like
        list of U, v and w, [(U1,v1,w1),(U2,v2,w2)...], v and w are optional
        
    Returns
    -------
    k1, uu[, vv[, ww, uw]] : 2-5 x array_like
        - k1: wave number
        - uu: The uu auto spectrum\n
        - vv: The vv auto spectrum, only if v is provided\n
        - ww: The ww auto spectrum, only if w is provided\n
        - uw: The uw cross spectrum, only if w is provided\n

        For two dimensional input, the mean spectra are returned
    """
    assert isinstance(sample_frq, (int, float))
    Uvw_arr = np.array(Uvw_lst)
    Ncomp = Uvw_arr.shape[1]
    U,v,w = [Uvw_arr[:,i,:].T for i in range(Ncomp)] + [None]*(3-Ncomp)
    k = 2 * np.pi * sample_frq / U.mean(0)
    
    if v is not None: assert np.abs(np.nanmean(v,0)).max()<1, "Max absolute mean of v is %f"%np.abs(np.nanmean(v,0)).max()
    if w is not None: assert np.abs(np.nanmean(w,0)).max()<1
    k1_vec = np.array([np.linspace(0, k_ / 2, U.shape[0] / 2)[1:] for k_ in k]).T
    u = U-np.nanmean(U, 0)
    u, v, w = detrend_wsp(u, v, w)

    return [k1_vec] + [spectrum(x1, x2, k=k) for x1, x2 in [(u, u), (v, v), (w, w), (w, u)]]


{'ns':( 298, 0.0455033341592, 30.7642421893, 2.32693322102, 0.00538254487239),
'nu':( 853, 0.0355868134195, 71.0190181051, 2.95576644546, 0.00268903502954),
's' :( 367, 0.0300454276184, 24.7375807569, 2.40165270878, 0.00253338624678),
'vs':( 223, 0.0149281609201, 11.7467239752, 3.21842435078, 0.00143453350246),
'n' :( 984, 0.0494970534618, 47.3846412548, 2.83466101838, 0.00513954022345),
'u' :( 696, 0.0258456660845, 78.9511607824, 2.51128584334, 0.00250014140782),
'vu':( 468, 0.0218274041889, 73.0730383576, 2.01636095317, 0.00196337613117)}

def bin_spectrum(x, y, bin_size, min_bin_count=2):
    assert min_bin_count > 0
    x = x / bin_size
    low, high = np.floor(np.nanmin(x)), np.ceil(np.nanmax(x))
    bins = int(high - low)
    nbr_in_bins = np.histogram(x, bins, range=(low, high))[0]
    if len(x.shape)==2:
        min_bin_count*=x.shape[1]
    mask = nbr_in_bins >= min_bin_count
    return np.histogram(x, bins, range=(low, high), weights=y)[0][mask] / nbr_in_bins[mask], nbr_in_bins


def logbin_spectrum(k1, xx, log10_bin_size=.2, min_bin_count=2):
    ln_bin_size = np.log(10) * log10_bin_size
    if xx is None:
        return None
    return (bin_spectrum(np.log(k1), (xx), ln_bin_size, min_bin_count)[0])


def logbin_spectra(k1, uu, vv=None, ww=None, uw=None, log10_bin_size=0.2, min_bin_count=2):
    return tuple([logbin_spectrum(k1, xx, log10_bin_size, min_bin_count) for xx in [k1, uu, vv, ww, uw]])

def plot_spectrum(spacial_frq, u, plt=None):
    if plt is None:
        import matplotlib.pyplot as plt
    k1, uu = logbin_spectra(*spectra(spacial_frq, u))[:2]

    plt.semilogx(k1, k1 * uu, 'b-')


def detrend_wsp(u, v=None, w=None):
    def _detrend(wsp):
        if wsp is None:
            return None
        dwsp = np.atleast_2d(wsp.copy().T).T
        t = np.arange(dwsp.shape[0])
        A = np.vstack([t, np.ones(len(t))]).T
        for i in range(dwsp.shape[1]):
            trend, offset = np.linalg.lstsq(A, dwsp[:, i])[0]
            dwsp[:, i] = dwsp[:, i] - t * trend + t[-1] / 2 * trend
        return dwsp.reshape(wsp.shape)
    return [_detrend(wsp) for wsp in [u, v, w]]


def plot_spectra(k1, uu, vv=None, ww=None, uw=None, mean_u=1, log10_bin_size=.2, plt=None, marker_style='.'):
    if plt is None:
        import matplotlib.pyplot as plt
    bk1, buu, bvv, bww, buw = logbin_spectra(k1, uu, vv, ww, uw, log10_bin_size)
    def plot(xx, label, color, plt):
        plt.semilogx(bk1, bk1 * xx * 10 ** 0 / mean_u ** 2 , marker_style + color, label=label)
    plot(buu, 'uu', 'r', plt)
    plt.xlabel('Wavenumber $k_{1}$ [$m^{-1}$]')
    if mean_u ==1:
        plt.ylabel(r'Spectral density $k_{1} F(k_{1}) [m^2/s^2]$')
    else:
        plt.ylabel(r'Spectral density $k_{1} F(k_{1})/U^{2} [-]$')
    if (bvv) is not None:
        plot(bvv, 'vv', 'g', plt)
    if bww is not None:
        plot(bww, 'ww', 'b', plt)
        plot(buw, 'uw', 'm', plt)


'''
Created on 07/02/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
import sys
from collections import OrderedDict
import os
standard_library.install_aliases()
import inspect
import numpy as np
def set_cache_property(obj, name, get_func, set_func=None):
    """Create a cached property

    Parameters
    ----------
    obj : object
        Class to add property to
    name : str
        Name of property
    get_func : func
        Getter function
    set_func : func, optional
        Setter function

    Examples
    --------
    >>> class Example(object):
    >>>     def __init__(self):
    >>>        set_cache_property(self, "test", self.slow_function)
    >>>
    >>> e = Example()
    >>> e.test # Call, store and return result of e.slow_function
    >>> e.test # Return stored result of e.slow_function
    >>> e._test = None # clear cache result
    >>> e.test # Call, store and return result of e.slow_function
    """
    _name = "_" + name
    setattr(obj, _name, None)
    def get(self):
        if getattr(obj, _name) is None:
            setattr(obj, _name, get_func())
        return getattr(obj, _name)

    p = property(lambda self:get(self), set_func)
    return setattr(obj.__class__, name, p)

def cache_function(f):
    """Cache function decorator

    Example:
    >>> class Example(object):
    >>>    @cache_function
    >>>    def slow_function(self):
    >>>        # calculate slow result
    >>>        return 1
    >>>
    >>> e = Example()
    >>> e.slow_function() # Call, store and return result of e.slow_function
    >>> e.slow_function() # Return stored result of e.slow_function
    """
    def wrap(*args, **kwargs):
        self = args[0]
        name = "_" + f.__name__
        if not hasattr(self, name) or getattr(self, name) is None or kwargs.get("reload", False):
            try:
                del kwargs['reload']
            except KeyError:
                pass
            # ======HERE============
            setattr(self, name, f(*args, **kwargs))
            # ======================
            if not hasattr(self, "cache_attr_lst"):
                self.cache_attr_lst = set()
                def clear_cache():
                    for attr in self.cache_attr_lst:
                        delattr(self, attr)
                    self.cache_attr_lst = set()
                self.clear_cache = clear_cache
            self.cache_attr_lst.add(name)

        return getattr(self, name)
    version = sys.version_info
    if version >= (3,3):
        if 'reload' in inspect.signature(f).parameters.values():
            raise AttributeError("Functions decorated with cache_function are not allowed to take a parameter called 'reload'")
    elif 'reload' in inspect.getargspec(f)[0]:
        raise AttributeError("Functions decorated with cache_function are not allowed to take a parameter called 'reload'")
    return wrap

class cache_method():
    def __init__(self, N):
        self.N = N
        
        
    def __call__(self, f):
        def wrapped(caller_obj, *args):
            name = "_" + f.__name__
            arg_id = ";".join([str(a) for a in args])
            if not hasattr(caller_obj,'%s_cache_dict'%name):
                setattr(caller_obj,'%s_cache_dict'%name, OrderedDict())
            cache_dict = getattr(caller_obj,'%s_cache_dict'%name)
            if arg_id not in cache_dict: 
                cache_dict[arg_id] = f(caller_obj, *args)
                if len(cache_dict)>self.N:
                    cache_dict.popitem(last=False)
            return cache_dict[arg_id]
        return wrapped
    
def cache_npsave(f):
    def wrap(filename,*args,**kwargs):
        np_filename = os.path.splitext(filename)[0] + ".npy"
        def loadsave():
            res = f(filename,*args,**kwargs)
            np.save(np_filename,res)
            return res
        if os.path.isfile(np_filename) and (not os.path.isfile(filename) or os.path.getmtime(np_filename) > os.path.getmtime(filename)):
            try:
                return np.load(np_filename)
            except:
                return loadsave()
        else:
            return loadsave()
    return wrap

def _get_npsavez_wrap(f, compress):
    def wrap(filename,*args,**kwargs):
        np_filename = os.path.splitext(filename)[0] + ".npy%s.npz"%("",".c")[compress]
        def loadsave():
            res = f(filename,*args,**kwargs)
            if compress:
                np.savez_compressed(np_filename,*res)
            else:
                np.savez(np_filename,*res)
            return res
        if os.path.isfile(np_filename) and (not os.path.isfile(filename) or os.path.getmtime(np_filename) > os.path.getmtime(filename)):
            try:
                npzfile = np.load(np_filename)
                return [npzfile['arr_%d'%i] for i in range(len(npzfile.files))]
            except:
                return loadsave()
        else:
            return loadsave()
    return wrap

def cache_npsavez(f):
    return _get_npsavez_wrap(f,False)


def cache_npsavez_compressed(f):
    return _get_npsavez_wrap(f, True)


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from builtins import str
import glob
from io import open
import json
import os
import re
import sys
from threading import Thread
import time

from future import standard_library
from wetb.hawc2 import log_file
from wetb.hawc2.htc_file import HTCFile, fmt_path
from wetb.hawc2.log_file import LogFile
import tempfile
import stat
import shutil


standard_library.install_aliases()

QUEUED = "queued"  # until start
PREPARING = "Copy to host"  # during prepare simulation
INITIALIZING = "Initializing"  # when starting
SIMULATING = "Simulating"  # when logfile.status=simulating
FINISHING = "Copy from host"  # during prepare simulation
FINISH = "Simulation finish"  # when HAWC2 finish
ERROR = "Error"  # when hawc2 returns error
ABORTED = "Aborted"  # when stopped and logfile.status != Done
CLEANED = "Cleaned"  # after copy back


class Simulation(object):
    """Class for doing hawc2 simulations




    Examples
    --------
    >>> sim = Simulation("<path>/MyModel","htc/MyHtc")

    Blocking inplace simulation
    >>> sim.simulate()

    Non-blocking distributed simulation(everything copied to temporary folder)\n
    Starts two threads:
    - non_blocking_simulation_thread:
        - prepare_simulation() # copy to temporary folder
        - simulate() # simulate
        - finish_simulation # copy results back again
    - updateSimStatusThread:
        - update status every second
    >>> sim.start()
    >>> while sim.status!=CLEANED:
    >>>     sim.show_status()


    The default host is LocalSimulationHost. To simulate on pbs featured cluster
    >>> sim.host = PBSClusterSimulationHost(sim, <hostname>, <username>, <password>)
    >>> sim.start()
    >>> while sim.status!=CLEANED:
    >>>     sim.show_status()
    """

    is_simulating = False
    is_done = False
    status = QUEUED

    def __init__(self, modelpath, htcfilename, hawc2exe="HAWC2MB.exe", copy_turbulence=True):
        self.modelpath = os.path.abspath(modelpath) + "/"
        if os.path.isabs(htcfilename):
            htcfilename = os.path.relpath(htcfilename, modelpath)
        if htcfilename.startswith("input/"):
            htcfilename = htcfilename[6:]
        exists = [os.path.isfile(os.path.join(modelpath, htcfilename)),
                  os.path.isfile(os.path.join(modelpath, "input/", htcfilename))]
        if all(exists):
            raise Exception(
                "Both standard and input/output file structure available for %s in %s. Delete one of the options" % (htcfilename, modelpath))
        if not any(exists):
            raise Exception("%s not found in %s" % (htcfilename, modelpath))
        else:
            self.ios = exists[1]  # input/output file structure

        if self.ios:
            self.exepath = self.modelpath + "input/"
        else:
            self.exepath = self.modelpath
        # model_path: top level path containing all resources
        # exepath: parent path for relative paths

        htcfilename = fmt_path(htcfilename)

        self.tmp_modelpath = self.exepath
        self.folder = os.path.dirname(htcfilename)

        self.filename = os.path.basename(htcfilename)
        self.htcFile = HTCFile(os.path.join(self.exepath, htcfilename), self.exepath)
        self.time_stop = self.htcFile.simulation.time_stop[0]
        self.hawc2exe = hawc2exe
        self.copy_turbulence = copy_turbulence
        self.simulation_id = (htcfilename.replace("\\", "/").replace("/", "_")[:50] + "_%d" % id(self))
        if self.simulation_id.startswith("input_"):
            self.simulation_id = self.simulation_id[6:]
        self.stdout_filename = fmt_path(os.path.join(os.path.relpath(self.exepath, self.modelpath),
                                                     (os.path.splitext(htcfilename)[0] + ".out").replace('htc', 'stdout', 1)))
        if self.ios:
            assert self.stdout_filename.startswith("input/")
            self.stdout_filename = self.stdout_filename.replace("input/", "../output/")
        # self.stdout_filename = "stdout/%s.out" % self.simulation_id
        if 'logfile' in self.htcFile.simulation:
            self.log_filename = self.htcFile.simulation.logfile[0]
        else:
            self.log_filename = self.stdout_filename
        if os.path.isabs(self.log_filename):
            self.log_filename = os.path.relpath(self.log_filename, self.modelpath)
        else:
            self.log_filename = os.path.relpath(self.log_filename)
        self.log_filename = fmt_path(self.log_filename)
        self.logFile = LogFile(os.path.join(self.exepath, self.log_filename), self.time_stop)
        self.logFile.clear()
        self.last_status = self.status
        self.errors = []
        self.non_blocking_simulation_thread = Thread(
            target=lambda: self.simulate_distributed(raise_simulation_errors=False))
        self.updateSimStatusThread = UpdateSimStatusThread(self)
        from wetb.hawc2.simulation_resources import LocalSimulationHost
        self.host = LocalSimulationHost(self)
        self.stdout = ""
        self.returncode = 0

    def start(self, update_interval=1):
        """Start non blocking distributed simulation"""
        self.is_simulating = True
        if update_interval > 0:
            self.updateSimStatusThread.interval = update_interval
            self.updateSimStatusThread.start()
        self.non_blocking_simulation_thread.start()

    def wait(self):
        self.non_blocking_simulation_thread.join()
        self.update_status()

    def abort(self, update_status=True):
        self.status = ABORTED
        self.is_simulating = False
        self.is_done = True
        self.host.stop()
        if update_status:
            self.update_status()

#         if self.status != QUEUED:
#             self.host.stop()
#             for _ in range(50):
#                 if self.is_simulating is False:
#                     break
#                 time.sleep(0.1)
#         if self.logFile.status not in [log_file.DONE]:
#             self.status = ABORTED
#         self.is_simulating = False
#         self.is_done = True
#         if update_status:
#             self.update_status()

    def show_status(self):
        # print ("log status:", self.logFile.status)
        if self.logFile.status == log_file.SIMULATING:
            if self.last_status != log_file.SIMULATING:
                print("|" + ("-" * 50) + "|" + ("-" * 49) + "|")
                sys.stdout.write("|")
            sys.stdout.write("." * (self.logFile.pct - getattr(self, 'last_pct', 0)))
            sys.stdout.flush()
            self.last_pct = self.logFile.pct
        elif self.last_status == log_file.SIMULATING:
            sys.stdout.write("." * (100 - self.last_pct) + "|")
            sys.stdout.flush()
            print("\n")
        elif self.logFile.status == log_file.UNKNOWN:
            print(self.status)
        else:
            print(self.logFile.status)
        if self.logFile.status != log_file.SIMULATING:
            if self.logFile.errors:
                print(self.logFile.errors)
        self.last_status = self.logFile.status

    def prepare_simulation(self):
        self.status = PREPARING
        # self.tmp_modelpath = os.path.join(".hawc2launcher/%s/" % self.simulation_id)
        # self.tmp_exepath = os.path.join(self.tmp_modelpath, os.path.relpath(self.exepath, self.modelpath) ) + "/"
        self.set_id(self.simulation_id, str(self.host), self.tmp_modelpath)

        def fmt(src):
            if os.path.isabs(src):
                src = os.path.relpath(os.path.abspath(src), self.exepath)
            else:
                src = os.path.relpath(src)
            assert not src.startswith(
                ".."), "%s referes to a file outside the model path\nAll input files be inside model path" % src
            return src
        if self.ios:
            input_folder_files = []
            for root, _, filenames in os.walk(os.path.join(self.modelpath, "input/")):
                for filename in filenames:
                    input_folder_files.append(os.path.join(root, filename))

            input_patterns = [fmt(src) for src in input_folder_files + ([], self.htcFile.turbulence_files())
                              [self.copy_turbulence] + self.additional_files().get('input', [])]
        else:
            input_patterns = [fmt(src) for src in self.htcFile.input_files(
            ) + ([], self.htcFile.turbulence_files())[self.copy_turbulence] + self.additional_files().get('input', [])]
        input_files = set([f for pattern in input_patterns for f in glob.glob(
            os.path.join(self.exepath, pattern)) if os.path.isfile(f) and ".hawc2launcher" not in f])

        self.host._prepare_simulation(input_files)


#        return [fmt(src) for src in self.htcFile.input_files() + self.htcFile.turbulence_files() + self.additional_files().get('input', [])]
#
#        for src in self._input_sources():
#            for src_file in glob.glob(os.path.join(self.modelpath, src)):
#
#
#        self.host._prepare_simulation()

    def simulate(self):
        # starts blocking simulation

        self.is_simulating = True
        self.errors = []
        self.status = INITIALIZING
        self.logFile.clear()
        self.host._simulate()
        self.returncode, self.stdout = self.host.returncode, self.host.stdout
        if self.host.returncode or 'error' in self.host.stdout.lower():
            if self.status == ABORTED:
                return
            if "error" in self.host.stdout.lower():
                self.errors = (list(set([l for l in self.host.stdout.split(
                    "\n") if 'error' in l.lower() and not 'rms error' in l])))
            self.status = ERROR
        if 'HAWC2MB version:' not in self.host.stdout:
            self.errors.append(self.host.stdout)
            self.status = ERROR

        self.logFile.update_status()
        self.errors.extend(list(set(self.logFile.errors)))
        self.update_status()
        self.is_simulating = False
        if self.host.returncode or self.errors:
            raise Exception("Simulation error:\nReturn code: %d\n%s" % (self.host.returncode, "\n".join(self.errors)))
        elif self.logFile.status != log_file.DONE or self.logFile.errors:
            raise Warning("Simulation succeded with errors:\nLog status:%s\nErrors:\n%s" %
                          (self.logFile.status, "\n".join(self.logFile.errors)))
        else:
            self.status = FINISH

    def finish_simulation(self):
        if self.status == ABORTED:
            return
        if self.status != ERROR:
            self.status = FINISHING

        def fmt(dst):
            if os.path.isabs(dst):
                dst = os.path.relpath(os.path.abspath(dst), self.exepath)
            else:
                dst = os.path.relpath(dst)
            dst = fmt_path(dst)
            assert not os.path.relpath(os.path.join(self.exepath, dst), self.modelpath).startswith(
                ".."), "%s referes to a file outside the model path\nAll input files be inside model path" % dst
            return dst
        turb_files = [f for f in self.htcFile.turbulence_files()
                      if self.copy_turbulence and not os.path.isfile(os.path.join(self.exepath, f))]
        if self.ios:
            output_patterns = ["../output/*", "../output/"] + turb_files + \
                [os.path.join(self.exepath, self.stdout_filename)]
        else:
            output_patterns = self.htcFile.output_files() + turb_files + \
                [os.path.join(self.exepath, self.stdout_filename)]
        output_files = set(self.host.glob([fmt_path(os.path.join(self.tmp_exepath, fmt(p)))
                                           for p in output_patterns], recursive=self.ios))
        if not os.path.isdir(os.path.dirname(self.exepath + self.stdout_filename)):
            os.makedirs(os.path.dirname(self.exepath + self.stdout_filename))

        try:
            self.host._finish_simulation(output_files)
            if self.status != ERROR:
                self.status = CLEANED
        except Warning as e:
            self.errors.append(str(e))
            if self.status != ERROR:
                self.status = CLEANED
        except Exception as e:
            self.errors.append(str(e))
            raise

        finally:
            self.set_id(self.filename)
            self.logFile.reset()
            self.htcFile.reset()

    def update_status(self, *args, **kwargs):
        self.host.update_logFile_status()
        if self.status in [INITIALIZING, SIMULATING]:
            if self.logFile.status == log_file.SIMULATING:
                self.status = SIMULATING
            if self.logFile.status == log_file.DONE and self.is_simulating is False:
                self.status = FINISH

    def __str__(self):
        return "Simulation(%s)" % self.filename

    def additional_files(self):
        additional_files_file = os.path.join(self.modelpath, 'additional_files.txt')
        additional_files = {}
        if os.path.isfile(additional_files_file):
            with open(additional_files_file, encoding='utf-8') as fid:
                additional_files = json.loads(fid.read().replace("\\", "/"))
        return additional_files

    def add_additional_input_file(self, file):
        additional_files = self.additional_files()
        additional_files['input'] = list(set(additional_files.get('input', []) + [file]))
        additional_files_file = os.path.join(self.modelpath, 'additional_files.txt')
        with open(additional_files_file, 'w', encoding='utf-8') as fid:
            json.dump(additional_files, fid)

    def simulate_distributed(self, raise_simulation_errors=True):
        try:
            tmpdirname = tempfile.TemporaryDirectory(prefix="h2launcher_%s_" % os.path.basename(self.filename)).name
            self.tmp_modelpath = tmpdirname + "/"
            self.tmp_exepath = os.path.join(self.tmp_modelpath, os.path.relpath(self.exepath, self.modelpath)) + "/"
            self.prepare_simulation()
            try:
                self.simulate()
            except Warning as e:
                print("simulation failed", str(self))
                print("Trying to finish")
                raise
            finally:
                try:
                    if self.status != ABORTED:
                        self.finish_simulation()
                except Exception:
                    print("finish_simulation failed", str(self))
                    raise
                finally:
                    def remove_readonly(fn, path, excinfo):
                        try:
                            os.chmod(path, stat.S_IWRITE)
                            fn(path)
                        except Exception as exc:
                            print("Skipped:", path, "because:\n", exc)

                    shutil.rmtree(tmpdirname, onerror=remove_readonly)

        except Exception as e:
            self.status = ERROR
            self.errors.append(str(e))
            if raise_simulation_errors:
                raise e
        finally:
            self.is_done = True

    def fix_errors(self):
        def confirm_add_additional_file(folder, file):
            if os.path.isfile(os.path.join(self.modelpath, folder, file)):
                filename = fmt_path(os.path.join(folder, file))
                if self.get_confirmation("File missing", "'%s' seems to be missing in the temporary working directory. \n\nDo you want to add it to additional_files.txt" % filename):
                    self.add_additional_input_file(filename)
                    self.show_message(
                        "'%s' is now added to additional_files.txt.\n\nPlease restart the simulation" % filename)
        for error in self.errors:
            for regex in [r".*\*\*\* ERROR \*\*\* File '(.*)' does not exist in the (.*) folder",
                          r".*\*\*\* ERROR \*\*\* DLL (.*)()"]:
                m = re.compile(regex).match(error.strip())
                if m is not None:
                    file, folder = m.groups()
                    confirm_add_additional_file(folder, file)
                    continue
            m = re.compile(r".*\*\*\* ERROR \*\*\* File '(.*)' does not exist in the working directory").match(error.strip())
            if m is not None:
                file = m.groups()[0]
                for root, folder, files in os.walk(self.modelpath):
                    if "__Thread" not in root and file in files:
                        folder = os.path.relpath(root, self.modelpath)
                        confirm_add_additional_file(folder, file)
                continue

    def get_confirmation(self, title, msg):
        """override in subclass"""
        return True

    def show_message(self, msg, title="Information"):
        print(msg)

    def set_id(self, *args, **kwargs):
        pass

    def progress_callback(self, *args, **kwargs):
        pass


class UpdateSimStatusThread(Thread):
    def __init__(self, simulation, interval=1):
        Thread.__init__(self)
        self.simulation = simulation
        self.interval = interval

    def start(self):
        Thread.start(self)

    def run(self):
        while self.simulation.is_done is False:
            self.simulation.update_status()
            time.sleep(0.5)
            t = time.time()
            while self.simulation.is_simulating and time.time() < t + self.interval:
                time.sleep(1)



'''
Created on 05/11/2015

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import unittest
import io
import os
import tempfile

import numpy as np
import pandas as pd

from wetb.prepost import windIO


class TestsLogFile(unittest.TestCase):

    def setUp(self):
        self.logpath = os.path.join(os.path.dirname(__file__),
                                    '../../hawc2/tests/test_files/logfiles/')

    def readlog(self, fname):
        log = windIO.LogFile()
        log.readlog(os.path.join(self.logpath, fname))
        return log

    def test_reading(self):
        fname = 'simulating.log'
        log = self.readlog(fname)
        self.assertTrue(hasattr(log, 'MsgListLog'))
        self.assertTrue(hasattr(log, 'MsgListLog2'))
        fpath = os.path.join(self.logpath, fname)
        self.assertEqual(len(log.MsgListLog), 1)
        self.assertEqual(len(log.MsgListLog2), 1)
        self.assertEqual(log.MsgListLog[0][0], fpath)
        self.assertTrue(fpath in log.MsgListLog2)
        # the current log file doesn't contain any errors but didn't complete
        # [found_error, exit_correct]
        self.assertEqual(log.MsgListLog2[fpath], [False, False])

    def test_loganalysis_file(self):
        fname = 'simulating.log'
        log = self.readlog(fname)
        csv = log._header()
        csv = log._msglistlog2csv(csv)
        # because our API is really crappy, we emulate writing to StringIO
        # instead of to a file
        fcsv = io.StringIO(csv)
        df = log.csv2df(fcsv)
        self.assertEqual(df.loc[0,'nr_time_steps'], 25)
        self.assertEqual(df.loc[0,'total_iterations'], 49)
        self.assertEqual(df.loc[0,'file_name'], log.MsgListLog[0][0])
        self.assertAlmostEqual(df.loc[0,'last_time_step'], 0.5, places=5)
        self.assertAlmostEqual(df.loc[0,'dt'], 0.02)
        self.assertAlmostEqual(df.loc[0,'max_iters_p_time_step'], 2.0)
        self.assertAlmostEqual(df.loc[0,'mean_iters_p_time_step'], 1.96)
        self.assertTrue(np.isnan(df.loc[0,'seconds_p_iteration']))

    def test_read_and_analysis_simulation_error2(self):

        fname = 'simulation_error2.log'
        fpath = os.path.join(self.logpath, fname)

        log = self.readlog(fname)
        # finish correctly, but with errors, [found_error, exit_correct]
        self.assertEqual(log.MsgListLog2[fpath], [True, True])

        csv = log._header()
        csv = log._msglistlog2csv(csv)
        # because our API is really crappy, we emulate writing to StringIO
        # instead of to a file
        fcsv = io.StringIO(csv)
        df = log.csv2df(fcsv)
        self.assertEqual(df.loc[0,'nr_time_steps'], 1388)
        self.assertEqual(df.loc[0,'total_iterations'], 0)
        self.assertEqual(df.loc[0,'file_name'], log.MsgListLog[0][0])
        self.assertAlmostEqual(df.loc[0,'dt'], 0.02)
        self.assertAlmostEqual(df.loc[0,'max_iters_p_time_step'], 0.0)
        self.assertAlmostEqual(df.loc[0,'mean_iters_p_time_step'], 0.0)
        self.assertAlmostEqual(df.loc[0,'elapsted_time'], 0.3656563)
        self.assertAlmostEqual(df.loc[0,'last_time_step'], 27.76, places=5)
        self.assertAlmostEqual(df.loc[0,'real_sim_time'], 75.9183, places=4)
        self.assertTrue(np.isnan(df.loc[0,'seconds_p_iteration']))

        self.assertEqual(df.loc[0,'first_tstep_104'], 1386)
        self.assertEqual(df.loc[0,'last_step_104'], 1388)
        self.assertEqual(df.loc[0,'nr_104'], 30)
        msg = ' *** ERROR *** Out of limits in user defined shear field - '
        msg += 'limit value used'
        self.assertEqual(df.loc[0,'msg_104'], msg)

    def test_read_and_analysis_init_error(self):

        fname = 'init_error.log'
        fpath = os.path.join(self.logpath, fname)

        log = self.readlog(fname)
        # errors, but no sim time or finish message [found_error, exit_correct]
        self.assertEqual(log.MsgListLog2[fpath], [True, True])

        csv = log._header()
        csv = log._msglistlog2csv(csv)
        # because our API is really crappy, we emulate writing to StringIO
        # instead of to a file
        fcsv = io.StringIO(csv)
        df = log.csv2df(fcsv)

        msg = ' *** ERROR *** No line termination in command line            8'
        self.assertEqual(df.loc[0,'msg_5'], msg)

    def test_read_and_analysis_init(self):

        fname = 'init.log'
        fpath = os.path.join(self.logpath, fname)

        log = self.readlog(fname)
        # errors, but no sim time or finish message [found_error, exit_correct]
        self.assertEqual(log.MsgListLog2[fpath], [False, False])

        csv = log._header()
        csv = log._msglistlog2csv(csv)
        # because our API is really crappy, we emulate writing to StringIO
        # instead of to a file
        fcsv = io.StringIO(csv)
        df = log.csv2df(fcsv)

    def test_read_and_analysis_tmp(self):

        fname = 'tmp.log'
        fpath = os.path.join(self.logpath, fname)

        log = self.readlog(fname)
        csv = log._header()
        csv = log._msglistlog2csv(csv)
        fcsv = io.StringIO(csv)
        df = log.csv2df(fcsv)
        # finish correctly, but with errors
        self.assertAlmostEqual(df.loc[0,'elapsted_time'], 291.6350, places=5)
        self.assertEqual(log.MsgListLog2[fpath], [True, True])


class TestsLoadResults(unittest.TestCase):

    def setUp(self):
        self.respath = os.path.join(os.path.dirname(__file__),
                                    '../../hawc2/tests/test_files/hawc2io/')
        self.fascii = 'Hawc2ascii'
        self.fbin = 'Hawc2bin'
        self.f1_chant = 'hawc2ascii_chantest_1.sel'
        self.f2_chant = 'hawc2bin_chantest_2.sel'
        self.f3_chant = 'hawc2bin_chantest_3.sel'

    def loadresfile(self, resfile):
        res = windIO.LoadResults(self.respath, resfile)
        self.assertTrue(hasattr(res, 'sig'))
        self.assertEqual(res.Freq, 40.0)
        self.assertEqual(res.N, 800)
        self.assertEqual(res.Nch, 28)
        self.assertEqual(res.Time, 20.0)
        self.assertEqual(res.sig.shape, (800, 28))
        return res

    def test_load_ascii(self):
        res = self.loadresfile(self.fascii)
        self.assertEqual(res.FileType, 'ASCII')

    def test_load_binary(self):
        res = self.loadresfile(self.fbin)
        self.assertEqual(res.FileType, 'BINARY')

    def test_compare_ascii_bin(self):
        res_ascii = windIO.LoadResults(self.respath, self.fascii)
        res_bin = windIO.LoadResults(self.respath, self.fbin)

        for k in range(res_ascii.sig.shape[1]):
            np.testing.assert_allclose(res_ascii.sig[:,k], res_bin.sig[:,k],
                                       rtol=1e-02, atol=0.001)

    def test_unified_chan_names(self):
        res = windIO.LoadResults(self.respath, self.fascii, readdata=False)
        self.assertFalse(hasattr(res, 'sig'))

        np.testing.assert_array_equal(res.ch_df.index.values, np.arange(0,28))
        self.assertEqual(res.ch_df.unique_ch_name.values[0], 'Time')
        self.assertEqual(res.ch_df.unique_ch_name.values[27],
                         'windspeed-global-Vy--2.50-1.00--52.50')

    def test_unified_chan_names_extensive(self):

        # ---------------------------------------------------------------------
        res = windIO.LoadResults(self.respath, self.f1_chant, readdata=False)
        self.assertFalse(hasattr(res, 'sig'))
        np.testing.assert_array_equal(res.ch_df.index.values, np.arange(0,425))
        self.assertEqual(res.ch_df.unique_ch_name.values[0], 'Time')
        df = res.ch_df
        self.assertEqual(2, len(df[df['bearing_name']=='shaft_rot']))
        self.assertEqual(18, len(df[df['sensortype']=='State pos']))
        self.assertEqual(11, len(df[df['blade_nr']==1]))

        exp = [[38, 'global-blade2-elem-019-zrel-1.00-State pos-z', 'm'],
               [200, 'blade2-blade2-node-017-momentvec-z', 'kNm'],
               [296, 'blade1-blade1-node-008-forcevec-z', 'kN'],
               [415, 'Cl-1-54.82', 'deg'],
               [421, 'qwerty-is-azerty', 'is'],
               [422, 'wind_wake-wake_pos_x_1', 'm'],
               [423, 'wind_wake-wake_pos_y_2', 'm'],
               [424, 'wind_wake-wake_pos_z_5', 'm'],
              ]
        for k in exp:
            self.assertEqual(df.loc[k[0], 'unique_ch_name'], k[1])
            self.assertEqual(df.loc[k[0], 'units'], k[2])
            self.assertEqual(res.ch_dict[k[1]]['chi'], k[0])
            self.assertEqual(res.ch_dict[k[1]]['units'], k[2])

        # ---------------------------------------------------------------------
        res = windIO.LoadResults(self.respath, self.f2_chant, readdata=False)
        self.assertFalse(hasattr(res, 'sig'))
        np.testing.assert_array_equal(res.ch_df.index.values, np.arange(0,217))
        df = res.ch_df
        self.assertEqual(4, len(df[df['sensortype']=='wsp-global']))
        self.assertEqual(2, len(df[df['sensortype']=='harmonic']))
        self.assertEqual(2, len(df[df['blade_nr']==3]))

        # ---------------------------------------------------------------------
        res = windIO.LoadResults(self.respath, self.f3_chant, readdata=False)
        self.assertFalse(hasattr(res, 'sig'))
        np.testing.assert_array_equal(res.ch_df.index.values, np.arange(0,294))
        df1 = res.ch_df
        self.assertEqual(8, len(df1[df1['sensortype']=='CT']))
        self.assertEqual(8, len(df1[df1['sensortype']=='CQ']))
        self.assertEqual(8, len(df1[df1['sensortype']=='a_grid']))
        self.assertEqual(84, len(df1[df1['blade_nr']==1]))

        fname = os.path.join(self.respath, self.f3_chant.replace('.sel',
                                                                 '.ch_df.csv'))
        # FIXME: read_csv for older pandas versions fails on reading the
        # mixed str/tuple column. Ignore the pos column for now
        colref = ['azimuth', 'bearing_name', 'blade_nr', 'bodyname',
                  'component', 'coord', 'direction', 'dll', 'flap_nr', 'io',
                  'io_nr', 'output_type', 'radius', 'sensortag',
                  'sensortype', 'unique_ch_name', 'units'] # 'pos',
        # keep_default_na: leave empyt strings as empty strings and not nan's
        # you can't have nice things: usecols in combination with index_col
        # doesn't work
        df2 = pd.read_csv(fname, usecols=['chi']+colref, keep_default_na=False)
        df2.index = df2['chi']
        df2.drop(labels='chi', inplace=True, axis=1)

        # for the comparison we need to have the columns with empty/number
        # mixed data types in a consistent data type
        for col in ['azimuth', 'radius', 'blade_nr', 'io_nr', 'flap_nr', 'dll']:
            df1.loc[df1[col]=='', col] = np.nan
            df1[col] = df1[col].astype(np.float32)
            df2.loc[df2[col]=='', col] = np.nan
            df2[col] = df2[col].astype(np.float32)

        # print(df1.pos[14], df2.pos[14])
        # the pos columns contains also tuples, read from csv doesn't get that
        # df1['pos'] = df1['pos'].astype(np.str)
        # df1['pos'] = df1['pos'].str.replace("'", "")
        # print(df1.pos[14], df2.pos[14])

        # sort columns in the same way so we can assert the df are equal
        # df1 = df1[colref].copy()
        # df2 = df2[colref].copy()
        # FIXME: when pandas is more recent we can use assert_frame_equal
        # pd.testing.assert_frame_equal(df1, df2)
        # ...but there is no testing.assert_frame_equal in pandas 0.14
        for col in colref:
            np.testing.assert_array_equal(df1[col].values, df2[col].values)


class TestUserWind(unittest.TestCase):

    def setUp(self):
        self.path = os.path.join(os.path.dirname(__file__), 'data')
        self.z_h = 100.0
        self.r_blade_tip = 50.0
        self.h_ME = 650
        self.z = np.array([self.z_h - self.r_blade_tip,
                           self.z_h + self.r_blade_tip])

    def test_deltaphi2aphi(self):

        uwind = windIO.UserWind()
        profiles = windIO.WindProfiles

        for a_phi_ref in [-1.0, 0.0, 0.5]:
            phis = profiles.veer_ekman_mod(self.z, self.z_h, h_ME=self.h_ME,
                                           a_phi=a_phi_ref)
            d_phi_ref = phis[1] - phis[0]
#            a_phi1 = uwind.deltaphi2aphi(d_phi_ref, self.z_h, self.r_blade_tip,
#                                         h_ME=self.h_ME)
            a_phi2 = uwind.deltaphi2aphi_opt(d_phi_ref, self.z, self.z_h,
                                             self.r_blade_tip, self.h_ME)
            self.assertAlmostEqual(a_phi_ref, a_phi2)

    def test_usershear(self):

        uwind = windIO.UserWind()

        # phi, shear, wdir
        combinations = [[1,0,0], [0,-0.2,0], [0,0,-10], [None, None, None],
                        [0.5,0.2,10]]

        for a_phi, shear, wdir in combinations:
            rpl = (a_phi, shear, wdir)
            try:
                fname = 'a_phi_%1.05f_shear_%1.02f_wdir%02i.txt' % rpl
            except:
                fname = 'a_phi_%s_shear_%s_wdir%s.txt' % rpl
            target = os.path.join(self.path, fname)

            fid = tempfile.NamedTemporaryFile(delete=False, mode='wb')
            target = os.path.join(self.path, fname)
            uu, vv, ww, xx, zz = uwind(self.z_h, self.r_blade_tip, a_phi=a_phi,
                                       nr_vert=5, nr_hor=3, h_ME=650.0,
                                       wdir=wdir, io=fid, shear_exp=shear)
            # FIXME: this has to be done more clean and Pythonic
            # load again for comparison with the reference
            uwind.fid.close()
            with open(uwind.fid.name) as fid:
                contents = fid.readlines()
            os.remove(uwind.fid.name)
            with open(target) as fid:
                ref = fid.readlines()
            self.assertEqual(contents, ref)


if __name__ == "__main__":
    unittest.main()




import numpy as np
import numpy.ma as ma
#from pylab import *
def interpolate(x, xp, yp, max_xp_step=None, max_dydxp=None, cyclic_range=None, max_repeated=None):
    """Interpolation similar to numpy.interp that handles nan and missing values

    Parameters
    ----------
    x : array_like
        The x-coordinates of the interpolated values.
    xp : 1-D sequence of floats
        The x-coordinates of the data points, must be increasing.
    yp : 1-D sequence of floats
        The y-coordinates of the data points, same length as xp.
    max_xp_step : int, float or None, optional
        Maximum xp-time step that is interpolated to x.\n
        If time step > max_xp_step then NAN is returned for intermediate x values
        If None, default, then this fix is not applied
    max_dydxp : int, float, None, optional
        Maximum absolute dydxp (slop of yp) that is interpolated to y.\n
        If dydxp > max_dydxp then NAN is returned for intermediate y values
        If None, default, then this fix is not applied
    cyclick_range : int, float, None, optional
        Range of posible values, e.g. 360 for degrees (both 0..360 and -180..180)
        If None (default), data not interpreted as cyclic
    max_repeated : int, float, None, optional
        Maximum xp that yp are allowed to be repeated
        if yp[i]==yp[i+1]==..==yp[i+j] and xp[i+j]-xp[i]>max_repeated_yp then
        NAN is returned for xp[i]<x<=xp[i+j]


    Returns
    -------
    y : {float, ndarray}
        The interpolated values, same shape as x.

    Examples
    --------
    >>> interpolate(x=[1,1.5,2,3], xp=[0.5,1.5,3], yp=[5,15,30])
    [10,15,20,30]
    >>> interpolate(x=[0, 1, 2, 7, 12], xp=[0, 2, 12], yp=[359, 0, 350], max_dydxp=45)
    [359,nan,0,175,350]
    """

    xp = np.array(xp, dtype=np.float)
    yp = np.array(yp, dtype=np.float)
    assert xp.shape[0] == yp.shape[0], "xp and yp must have same length (%d!=%d)" % (xp.shape[0], yp.shape[0])
    non_nan = ~(np.isnan(xp) & np.isnan(yp))
    yp = yp[non_nan]
    xp = xp[non_nan]
    y = np.interp(x, xp, yp, np.nan, np.nan)


    if cyclic_range is not None:
        cr = cyclic_range
        y2 = np.interp(x, xp, (yp + cr / 2) % cr - cr / 2, np.nan, np.nan) % cr
        y = np.choose(np.r_[0, np.abs(np.diff(y)) > np.abs(np.diff(y2))], np.array([y, y2]))

    if max_xp_step:
        diff = np.diff(xp)
        diff[np.isnan(diff)] = 0

        indexes = (np.where(diff > max_xp_step)[0])
        for index in indexes:
            y[(x > xp[index]) & (x < xp[index + 1])] = np.nan
    if max_dydxp:
        if cyclic_range is None:
            abs_dydxp = np.abs(np.diff(yp) / np.diff(xp))
        else:
            abs_dydxp = np.min([np.abs(np.diff((yp + cyclic_range / 2) % cyclic_range)) , np.abs(np.diff(yp % cyclic_range)) ], 0) / np.diff(xp)
        abs_dydxp[np.isnan(abs_dydxp)] = 0

        indexes = (np.where(abs_dydxp > max_dydxp)[0])
        for index in indexes:
            y[(x > xp[index]) & (x < xp[index + 1])] = np.nan
    if max_repeated:
        rep = np.r_[False, yp[1:] == yp[:-1], False]
        tr = rep[1:] ^ rep[:-1]
        itr = np.where(tr)[0]
        for start, stop, l in zip (itr[::2] , itr[1::2], xp[itr[1::2]] - xp[itr[::2]]):
            if l >= max_repeated:
                y[(x > xp[start]) & (x <= xp[stop])] = np.nan
    return y
#print (interpolate(x=[0, 1, 2, 3, 4], xp=[0, 1, 2, 4], yp=[5, 5, 6, 6], max_repeated=1))



'''
Created on 19/07/2016

@author: MMPE
'''
'''
Created on 02/11/2015

@author: MMPE
'''

import numpy as np
def _begin_end(x, mask):
    begin = np.where(~mask[:-1] & mask[1:])[0]
    end = np.where(mask[:-1] & ~mask[1:])[0] + 1
    if begin[0] > end[0]:
        x[:end[0]] = x[end[0]]
        end = end[1:]
    if begin[-1] > end[-1]:
        x[begin[-1]:] = x[begin[-1]]
        begin = begin[:-1]
    return begin, end

def replace_by_nan(data, spike_mask):
    data[spike_mask] = np.nan
    return data

def replace_by_mean(x, mask):
    x = x.copy()

    for b, e in zip(*_begin_end(x, mask)):
        x[b + 1:e] = np.mean([x[b], x[e]])
    return x

def replace_by_line(x, mask):
    x = x.copy()
    for b, e in zip(*_begin_end(x, mask)):
        x[b + 1:e] = np.interp(np.arange(b + 1, e), [b, e], [x[b], x[e]])
    return x

def replace_by_polynomial(x, mask, deg=3, no_base_points=12):
    if not np.any(mask):
        return x
    x = x.copy()
    begin, end = _begin_end(x, mask)
    for last_e, b, e, next_b in zip(np.r_[0, end[:-1]], begin, end, np.r_[begin[1:], len(x)]):
        if  b - last_e >= no_base_points and next_b - e >= no_base_points:
            pbegin, pend = max(b - no_base_points, last_e), min(e + no_base_points, next_b)
            inter_points = np.r_[np.arange(pbegin + 1, b + 1), np.arange(e, pend)]
            pol = np.poly1d(np.polyfit(inter_points , x[inter_points], deg))(np.arange(b + 1, e))
            if pol.max() - pol.min() > 2 * (x[inter_points].max() - x[inter_points].min()):
                # use linear interpolation if range is too big
                x[b + 1:e] = np.interp(np.arange(b + 1, e), [b, e], [x[b], x[e]])
            else:
                x[b + 1:e] = pol
        else:
            x[b + 1:e] = np.interp(np.arange(b + 1, e), [b, e], [x[b], x[e]])
    return x

def max_cont_mask_length(mask):
    if not np.any(mask):
        return 0
    begin = np.where(~mask[:-1] & mask[1:])[0]
    end = np.where(mask[:-1] & ~mask[1:])[0] + 1
    no_nan = []
    if begin[0] > end[0]:
        no_nan.append(end[0])
        end = end[1:]
    if begin[-1] > end[-1]:
        no_nan.append(len(mask) - begin[-1] - 1)
        begin = begin[:-1]
    return max(np.r_[no_nan, end - 1 - begin])





'''
Created on 06/11/2015

@author: MMPE
'''


def wsp_mask(x, wsp, pm_tolerance):
    return (wsp - pm_tolerance <= x) & (x < wsp + pm_tolerance)

def wdir_mask(x, wdir, pm_tolerance):
    return ((x - wdir + pm_tolerance) % 360 >= 0) & ((x - wdir + pm_tolerance) % 360 < 2 * pm_tolerance)

